@@ -1,6 +1,6 @@
 /*
  * blocklist.c
- * Copyright (C) 1998-2000 A.J. van Os; Released under GPL
+ * Copyright (C) 1998-2001 A.J. van Os; Released under GPL
  *
  * Description:
  * Build, read and destroy a list of Word text blocks
@@ -11,7 +11,7 @@
 
 
 /*
- * Privat structure to hide the way the information
+ * Private structure to hide the way the information
  * is stored from the rest of the program
  */
 typedef struct list_mem_tag {
@@ -25,7 +25,7 @@ static list_mem_type	*pFootAnchor = NULL;
 static list_mem_type	*pUnused1Anchor = NULL;
 static list_mem_type	*pEndAnchor = NULL;
 static list_mem_type	*pUnused2Anchor = NULL;
-/* Variable needed to read the block list */
+/* Variable needed to build the block list */
 static list_mem_type	*pBlockLast = NULL;
 /* Variable needed to read the text block list */
 static list_mem_type	*pTextBlockCurrent = NULL;
@@ -73,6 +73,7 @@ vDestroyTextBlockList(void)
 	pBlockLast = NULL;
 	pTextBlockCurrent = NULL;
 	pFootBlockCurrent = NULL;
+	pEndBlockCurrent = NULL;
 } /* end of vDestroyTextBlockList */
 
 /*
@@ -88,30 +89,36 @@ bAdd2TextBlockList(text_block_type *pTextBlock)
 	fail(pTextBlock == NULL);
 	fail(pTextBlock->lFileOffset < 0);
 	fail(pTextBlock->lTextOffset < 0);
-	fail(pTextBlock->tLength == 0);
-	fail(pTextBlock->bUsesUnicode && odd(pTextBlock->tLength));
+	fail(pTextBlock->lLength <= 0);
+	fail(pTextBlock->bUsesUnicode && odd(pTextBlock->lLength));
 
 	NO_DBG_MSG("bAdd2TextBlockList");
 	NO_DBG_HEX(pTextBlock->lFileOffset);
 	NO_DBG_HEX(pTextBlock->lTextOffset);
-	NO_DBG_HEX(pTextBlock->tLength);
+	NO_DBG_HEX(pTextBlock->lLength);
 	NO_DBG_DEC(pTextBlock->bUsesUnicode);
+	NO_DBG_DEC(pTextBlock->usPropMod);
 
 	if (pTextBlock->lFileOffset < 0 ||
 	    pTextBlock->lTextOffset < 0 ||
-	    pTextBlock->tLength == 0) {
+	    pTextBlock->lLength <= 0 ||
+	    (pTextBlock->bUsesUnicode && odd(pTextBlock->lLength))) {
 		werr(0, "Software (textblock) error");
 		return FALSE;
 	}
-	/* Check for continuous blocks of the same character size */
+	/*
+	 * Check for continuous blocks of the same character size and
+	 * the same properties modifier
+	 */
 	if (pBlockLast != NULL &&
 	    pBlockLast->tInfo.lFileOffset +
-	     (long)pBlockLast->tInfo.tLength == pTextBlock->lFileOffset &&
+	     pBlockLast->tInfo.lLength == pTextBlock->lFileOffset &&
 	    pBlockLast->tInfo.lTextOffset +
-	     (long)pBlockLast->tInfo.tLength == pTextBlock->lTextOffset &&
-	    pBlockLast->tInfo.bUsesUnicode == pTextBlock->bUsesUnicode) {
+	     pBlockLast->tInfo.lLength == pTextBlock->lTextOffset &&
+	    pBlockLast->tInfo.bUsesUnicode == pTextBlock->bUsesUnicode &&
+	    pBlockLast->tInfo.usPropMod == pTextBlock->usPropMod) {
 		/* These are continous blocks */
-		pBlockLast->tInfo.tLength += pTextBlock->tLength;
+		pBlockLast->tInfo.lLength += pTextBlock->lLength;
 		return TRUE;
 	}
 	/* Make a new block */
@@ -131,88 +138,89 @@ bAdd2TextBlockList(text_block_type *pTextBlock)
 
 
 /*
- * vSplitBlockList - split the block list in three parts
+ * vSplitBlockList - split the block list in five parts
  *
  * Split the blocklist in a Text block list, a Footnote block list, a
  * Endnote block list and two Unused lists.
  *
  * NOTE:
- * The various i*Len input parameters are given in characters, but the
+ * The various l*Len input parameters are given in characters, but the
  * length of the blocks are in bytes.
  */
 void
-vSplitBlockList(size_t tTextLen, size_t tFootnoteLen, size_t tUnused1Len,
-	size_t tEndnoteLen, size_t tUnused2Len, BOOL bMustExtend)
+vSplitBlockList(long lTextLen, long lFootnoteLen, long lUnused1Len,
+	long lEndnoteLen, long lUnused2Len, BOOL bMustExtend)
 {
 	list_mem_type	*apAnchors[5];
 	list_mem_type	*pGarbageAnchor, *pCurr, *pNext;
-	int		iIndex, iCharsToGo, iBytesTooFar;
+	long		lCharsToGo, lBytesTooFar;
+	int		iIndex;
 #if defined(DEBUG)
-	size_t		tTotal;
+	long		lTotal;
 #endif /* DEBUG */
 
 	DBG_MSG("vSplitBlockList");
 
 /* Text block list */
 	pCurr = NULL;
-	iCharsToGo = (int)tTextLen;
-	iBytesTooFar = -1;
-	if (tTextLen != 0) {
+	lCharsToGo = lTextLen;
+	lBytesTooFar = -1;
+	if (lTextLen > 0) {
 		DBG_MSG("Text block list");
-		DBG_DEC(tTextLen);
+		DBG_DEC(lTextLen);
 		for (pCurr = pTextAnchor;
 		     pCurr != NULL;
 		     pCurr = pCurr->pNext) {
-			NO_DBG_DEC(pCurr->tInfo.tLength);
-			fail(pCurr->tInfo.tLength == 0);
+			NO_DBG_DEC(pCurr->tInfo.lLength);
+			fail(pCurr->tInfo.lLength <= 0);
 			if (pCurr->tInfo.bUsesUnicode) {
-				fail(odd(pCurr->tInfo.tLength));
-				iCharsToGo -= (int)(pCurr->tInfo.tLength / 2);
-				if (iCharsToGo < 0) {
-					iBytesTooFar = -2 * iCharsToGo;
+				fail(odd(pCurr->tInfo.lLength));
+				lCharsToGo -= pCurr->tInfo.lLength / 2;
+				if (lCharsToGo < 0) {
+					lBytesTooFar = -2 * lCharsToGo;
 				}
 			} else {
-				iCharsToGo -= (int)pCurr->tInfo.tLength;
-				if (iCharsToGo < 0) {
-					iBytesTooFar = -iCharsToGo;
+				lCharsToGo -= pCurr->tInfo.lLength;
+				if (lCharsToGo < 0) {
+					lBytesTooFar = -lCharsToGo;
 				}
 			}
-			if (iCharsToGo <= 0) {
+			if (lCharsToGo <= 0) {
 				break;
 			}
 		}
 	}
 /* Split the list */
-	if (tTextLen == 0) {
+	if (lTextLen <= 0) {
 		/* Empty text blocks list */
 		pFootAnchor = pTextAnchor;
 		pTextAnchor = NULL;
 	} else if (pCurr == NULL) {
 		/* No footnote blocks */
 		pFootAnchor = NULL;
-	} else if (iCharsToGo == 0) {
+	} else if (lCharsToGo == 0) {
 		/* Move the integral number of footnote blocks */
 		pFootAnchor = pCurr->pNext;
 		pCurr->pNext = NULL;
 	} else {
 		/* Split the part-text block, part-footnote block */
-		DBG_DEC(iBytesTooFar);
-		fail(iBytesTooFar <= 0);
+		DBG_DEC(lBytesTooFar);
+		fail(lBytesTooFar <= 0);
 		pFootAnchor = xmalloc(sizeof(list_mem_type));
 		DBG_HEX(pCurr->tInfo.lFileOffset);
 		pFootAnchor->tInfo.lFileOffset =
 				pCurr->tInfo.lFileOffset +
-				(long)pCurr->tInfo.tLength -
-				iBytesTooFar;
+				pCurr->tInfo.lLength -
+				lBytesTooFar;
 		DBG_HEX(pFootAnchor->tInfo.lFileOffset);
 		DBG_HEX(pCurr->tInfo.lTextOffset);
 		pFootAnchor->tInfo.lTextOffset =
 				pCurr->tInfo.lTextOffset +
-				(long)pCurr->tInfo.tLength -
-				iBytesTooFar;
+				pCurr->tInfo.lLength -
+				lBytesTooFar;
 		DBG_HEX(pFootAnchor->tInfo.lTextOffset);
-		pFootAnchor->tInfo.tLength = (size_t)iBytesTooFar;
-		pCurr->tInfo.tLength -= (size_t)iBytesTooFar;
+		pFootAnchor->tInfo.lLength = lBytesTooFar;
+		pCurr->tInfo.lLength -= lBytesTooFar;
 		pFootAnchor->tInfo.bUsesUnicode = pCurr->tInfo.bUsesUnicode;
 		/* Move the integral number of footnote blocks */
 		pFootAnchor->pNext = pCurr->pNext;
@@ -220,64 +228,64 @@ vSplitBlockList(size_t tTextLen, size_t tFootnoteLen, size_t tUnused1Len,
 	}
 /* Footnote block list */
 	pCurr = NULL;
-	iCharsToGo = (int)tFootnoteLen;
-	iBytesTooFar = -1;
-	if (tFootnoteLen != 0) {
+	lCharsToGo = lFootnoteLen;
+	lBytesTooFar = -1;
+	if (lFootnoteLen > 0) {
 		DBG_MSG("Footnote block list");
-		DBG_DEC(tFootnoteLen);
+		DBG_DEC(lFootnoteLen);
 		for (pCurr = pFootAnchor;
 		     pCurr != NULL;
 		     pCurr = pCurr->pNext) {
-			DBG_DEC(pCurr->tInfo.tLength);
-			fail(pCurr->tInfo.tLength == 0);
+			DBG_DEC(pCurr->tInfo.lLength);
+			fail(pCurr->tInfo.lLength <= 0);
 			if (pCurr->tInfo.bUsesUnicode) {
-				fail(odd(pCurr->tInfo.tLength));
-				iCharsToGo -= (int)(pCurr->tInfo.tLength / 2);
-				if (iCharsToGo < 0) {
-					iBytesTooFar = -2 * iCharsToGo;
+				fail(odd(pCurr->tInfo.lLength));
+				lCharsToGo -= pCurr->tInfo.lLength / 2;
+				if (lCharsToGo < 0) {
+					lBytesTooFar = -2 * lCharsToGo;
 				}
 			} else {
-				iCharsToGo -= (int)pCurr->tInfo.tLength;
-				if (iCharsToGo < 0) {
-					iBytesTooFar = -iCharsToGo;
+				lCharsToGo -= pCurr->tInfo.lLength;
+				if (lCharsToGo < 0) {
+					lBytesTooFar = -lCharsToGo;
 				}
 			}
-			if (iCharsToGo <= 0) {
+			if (lCharsToGo <= 0) {
 				break;
 			}
 		}
 	}
 /* Split the list */
-	if (tFootnoteLen == 0) {
+	if (lFootnoteLen <= 0) {
 		/* Empty footnote list */
 		pUnused1Anchor = pFootAnchor;
 		pFootAnchor = NULL;
 	} else if (pCurr == NULL) {
 		/* No unused1 blocks */
 		pUnused1Anchor = NULL;
-	} else if (iCharsToGo == 0) {
+	} else if (lCharsToGo == 0) {
 		/* Move the integral number of unused1-list blocks */
 		pUnused1Anchor = pCurr->pNext;
 		pCurr->pNext = NULL;
 	} else {
 	  	/* Split the part-footnote block, part-unused1 block */
-		DBG_DEC(iBytesTooFar);
-		fail(iBytesTooFar <= 0);
+		DBG_DEC(lBytesTooFar);
+		fail(lBytesTooFar <= 0);
 		pUnused1Anchor = xmalloc(sizeof(list_mem_type));
 		DBG_HEX(pCurr->tInfo.lFileOffset);
 		pUnused1Anchor->tInfo.lFileOffset =
 				pCurr->tInfo.lFileOffset +
-				(long)pCurr->tInfo.tLength -
-				iBytesTooFar;
+				pCurr->tInfo.lLength -
+				lBytesTooFar;
 		DBG_HEX(pUnused1Anchor->tInfo.lFileOffset);
 		DBG_HEX(pCurr->tInfo.lTextOffset);
 		pUnused1Anchor->tInfo.lTextOffset =
 				pCurr->tInfo.lTextOffset +
-				(long)pCurr->tInfo.tLength -
-				iBytesTooFar;
+				pCurr->tInfo.lLength -
+				lBytesTooFar;
 		DBG_HEX(pUnused1Anchor->tInfo.lTextOffset);
-		pUnused1Anchor->tInfo.tLength = (size_t)iBytesTooFar;
-		pCurr->tInfo.tLength -= (size_t)iBytesTooFar;
+		pUnused1Anchor->tInfo.lLength = lBytesTooFar;
+		pCurr->tInfo.lLength -= lBytesTooFar;
 		pUnused1Anchor->tInfo.bUsesUnicode =
 				pCurr->tInfo.bUsesUnicode;
 		/* Move the integral number of unused1 blocks */
@@ -286,64 +294,64 @@ vSplitBlockList(size_t tTextLen, size_t tFootnoteLen, size_t tUnused1Len,
 	}
 /* Unused1 block list */
 	pCurr = NULL;
-	iCharsToGo = (int)tUnused1Len;
-	iBytesTooFar = -1;
-	if (tUnused1Len != 0) {
+	lCharsToGo = lUnused1Len;
+	lBytesTooFar = -1;
+	if (lUnused1Len > 0) {
 		DBG_MSG("Unused1 block list");
-		DBG_DEC(tUnused1Len);
+		DBG_DEC(lUnused1Len);
 		for (pCurr = pUnused1Anchor;
 		     pCurr != NULL;
 		     pCurr = pCurr->pNext) {
-			DBG_DEC(pCurr->tInfo.tLength);
-			fail(pCurr->tInfo.tLength == 0);
+			DBG_DEC(pCurr->tInfo.lLength);
+			fail(pCurr->tInfo.lLength <= 0);
 			if (pCurr->tInfo.bUsesUnicode) {
-				fail(odd(pCurr->tInfo.tLength));
-				iCharsToGo -= (int)(pCurr->tInfo.tLength / 2);
-				if (iCharsToGo < 0) {
-					iBytesTooFar = -2 * iCharsToGo;
+				fail(odd(pCurr->tInfo.lLength));
+				lCharsToGo -= pCurr->tInfo.lLength / 2;
+				if (lCharsToGo < 0) {
+					lBytesTooFar = -2 * lCharsToGo;
 				}
 			} else {
-				iCharsToGo -= (int)pCurr->tInfo.tLength;
-				if (iCharsToGo < 0) {
-					iBytesTooFar = -iCharsToGo;
+				lCharsToGo -= pCurr->tInfo.lLength;
+				if (lCharsToGo < 0) {
+					lBytesTooFar = -lCharsToGo;
 				}
 			}
-			if (iCharsToGo <= 0) {
+			if (lCharsToGo <= 0) {
 				break;
 			}
 		}
 	}
 /* Split the list */
-	if (tUnused1Len == 0) {
+	if (lUnused1Len <= 0) {
 		/* Empty unused1 list */
 		pEndAnchor = pUnused1Anchor;
 		pUnused1Anchor = NULL;
 	} else if (pCurr == NULL) {
 		/* No endnote blocks */
 		pEndAnchor = NULL;
-	} else if (iCharsToGo == 0) {
+	} else if (lCharsToGo == 0) {
 		/* Move the intergral number of endnote blocks */
 		pEndAnchor = pCurr->pNext;
 		pCurr->pNext = NULL;
 	} else {
 		/* Split the part-unused1-list block, part-endnote block */
-		DBG_DEC(iBytesTooFar);
-		fail(iBytesTooFar <= 0);
+		DBG_DEC(lBytesTooFar);
+		fail(lBytesTooFar <= 0);
 		pEndAnchor = xmalloc(sizeof(list_mem_type));
 		DBG_HEX(pCurr->tInfo.lFileOffset);
 		pEndAnchor->tInfo.lFileOffset =
 				pCurr->tInfo.lFileOffset +
-				(long)pCurr->tInfo.tLength -
-				iBytesTooFar;
+				pCurr->tInfo.lLength -
+				lBytesTooFar;
 		DBG_HEX(pEndAnchor->tInfo.lFileOffset);
 		DBG_HEX(pCurr->tInfo.lTextOffset);
 		pEndAnchor->tInfo.lTextOffset =
 				pCurr->tInfo.lTextOffset +
-				(long)pCurr->tInfo.tLength -
-				iBytesTooFar;
+				pCurr->tInfo.lLength -
+				lBytesTooFar;
 		DBG_HEX(pEndAnchor->tInfo.lTextOffset);
-		pEndAnchor->tInfo.tLength = (size_t)iBytesTooFar;
-		pCurr->tInfo.tLength -= (size_t)iBytesTooFar;
+		pEndAnchor->tInfo.lLength = lBytesTooFar;
+		pCurr->tInfo.lLength -= lBytesTooFar;
 		pEndAnchor->tInfo.bUsesUnicode = pCurr->tInfo.bUsesUnicode;
 		/* Move the integral number of endnote blocks */
 		pEndAnchor->pNext = pCurr->pNext;
@@ -351,64 +359,64 @@ vSplitBlockList(size_t tTextLen, size_t tFootnoteLen, size_t tUnused1Len,
 	}
 /* Endnote block list */
 	pCurr = NULL;
-	iCharsToGo = (int)tEndnoteLen;
-	iBytesTooFar = -1;
-	if (tEndnoteLen != 0) {
+	lCharsToGo = lEndnoteLen;
+	lBytesTooFar = -1;
+	if (lEndnoteLen > 0) {
 		DBG_MSG("Endnote block list");
-		DBG_DEC(tEndnoteLen);
+		DBG_DEC(lEndnoteLen);
 		for (pCurr = pEndAnchor;
 		     pCurr != NULL;
 		     pCurr = pCurr->pNext) {
-			DBG_DEC(pCurr->tInfo.tLength);
-			fail(pCurr->tInfo.tLength == 0);
+			DBG_DEC(pCurr->tInfo.lLength);
+			fail(pCurr->tInfo.lLength <= 0);
 			if (pCurr->tInfo.bUsesUnicode) {
-				fail(odd(pCurr->tInfo.tLength));
-				iCharsToGo -= (int)(pCurr->tInfo.tLength / 2);
-				if (iCharsToGo <= 0) {
-					iBytesTooFar = -2 * iCharsToGo;
+				fail(odd(pCurr->tInfo.lLength));
+				lCharsToGo -= pCurr->tInfo.lLength / 2;
+				if (lCharsToGo <= 0) {
+					lBytesTooFar = -2 * lCharsToGo;
 				}
 			} else {
-				iCharsToGo -= (int)pCurr->tInfo.tLength;
-				if (iCharsToGo <= 0) {
-					iBytesTooFar = -iCharsToGo;
+				lCharsToGo -= pCurr->tInfo.lLength;
+				if (lCharsToGo <= 0) {
+					lBytesTooFar = -lCharsToGo;
 				}
 			}
-			if (iCharsToGo <= 0) {
+			if (lCharsToGo <= 0) {
 				break;
 			}
 		}
 	}
 /* Split the list */
-	if (tEndnoteLen == 0) {
+	if (lEndnoteLen <= 0) {
 		/* Empty endnote list */
 		pUnused2Anchor = pEndAnchor;
 		pEndAnchor = NULL;
 	} else if (pCurr == NULL) {
 		/* No unused2 blocks */
 		pUnused2Anchor = NULL;
-	} else if (iCharsToGo == 0) {
+	} else if (lCharsToGo == 0) {
 		/* Move the intergral number of unused2 blocks */
 		pUnused2Anchor = pCurr->pNext;
 		pCurr->pNext = NULL;
 	} else {
 		/* Split the part-endnote block, part-unused2 block */
-		DBG_DEC(iBytesTooFar);
-		fail(iBytesTooFar <= 0);
+		DBG_DEC(lBytesTooFar);
+		fail(lBytesTooFar <= 0);
 		pUnused2Anchor = xmalloc(sizeof(list_mem_type));
 		DBG_HEX(pCurr->tInfo.lFileOffset);
 		pUnused2Anchor->tInfo.lFileOffset =
 				pCurr->tInfo.lFileOffset +
-				(long)pCurr->tInfo.tLength -
-				iBytesTooFar;
+				pCurr->tInfo.lLength -
+				lBytesTooFar;
 		DBG_HEX(pUnused2Anchor->tInfo.lFileOffset);
 		DBG_HEX(pCurr->tInfo.lTextOffset);
 		pUnused2Anchor->tInfo.lTextOffset =
 				pCurr->tInfo.lTextOffset +
-				(long)pCurr->tInfo.tLength -
-				iBytesTooFar;
+				pCurr->tInfo.lLength -
+				lBytesTooFar;
 		DBG_HEX(pUnused2Anchor->tInfo.lTextOffset);
-		pUnused2Anchor->tInfo.tLength = (size_t)iBytesTooFar;
-		pCurr->tInfo.tLength -= (size_t)iBytesTooFar;
+		pUnused2Anchor->tInfo.lLength = lBytesTooFar;
+		pCurr->tInfo.lLength -= lBytesTooFar;
 		pUnused2Anchor->tInfo.bUsesUnicode =
 				pCurr->tInfo.bUsesUnicode;
 		/* Move the integral number of unused2 blocks */
@@ -417,49 +425,50 @@ vSplitBlockList(size_t tTextLen, size_t tFootnoteLen, size_t tUnused1Len,
 	}
 /* Unused2 block list */
 	pCurr = NULL;
-	iCharsToGo = (int)tUnused2Len;
-	iBytesTooFar = -1;
-	if (tUnused2Len != 0) {
+	lCharsToGo = lUnused2Len;
+	lBytesTooFar = -1;
+	if (lUnused2Len > 0) {
 		DBG_MSG("Unused2 block list");
-		DBG_DEC(tUnused2Len);
+		DBG_DEC(lUnused2Len);
 		for (pCurr = pUnused2Anchor;
 		     pCurr != NULL;
 		     pCurr = pCurr->pNext) {
-			DBG_DEC(pCurr->tInfo.tLength);
-			fail(pCurr->tInfo.tLength == 0);
+			DBG_DEC(pCurr->tInfo.lLength);
+			fail(pCurr->tInfo.lLength <= 0);
 			if (pCurr->tInfo.bUsesUnicode) {
-				fail(odd(pCurr->tInfo.tLength));
-				iCharsToGo -= (int)(pCurr->tInfo.tLength / 2);
-				if (iCharsToGo < 0) {
-					iBytesTooFar = -2 * iCharsToGo;
+				fail(odd(pCurr->tInfo.lLength));
+				lCharsToGo -= pCurr->tInfo.lLength / 2;
+				if (lCharsToGo < 0) {
+					lBytesTooFar = -2 * lCharsToGo;
 				}
 			} else {
-				iCharsToGo -= (int)pCurr->tInfo.tLength;
-				if (iCharsToGo < 0) {
-					iBytesTooFar = -iCharsToGo;
+				lCharsToGo -= pCurr->tInfo.lLength;
+				if (lCharsToGo < 0) {
+					lBytesTooFar = -lCharsToGo;
 				}
 			}
-			if (iCharsToGo <= 0) {
+			if (lCharsToGo <= 0) {
 				break;
 			}
 		}
 	}
 /* Split the list */
-	if (tUnused2Len == 0) {
+	if (lUnused2Len <= 0) {
 		/* Empty unused2 list */
 		pGarbageAnchor = pUnused2Anchor;
 		pUnused2Anchor = NULL;
 	} else if (pCurr == NULL) {
 		/* No garbage block list */
 		pGarbageAnchor = NULL;
-	} else if (iCharsToGo == 0) {
+	} else if (lCharsToGo == 0) {
 		/* Move the intergral number of garbage blocks */
 		pGarbageAnchor = pCurr->pNext;
+		pCurr->pNext = NULL;
 	} else {
-		/* Reduce the part-endnote block */
-		DBG_DEC(iBytesTooFar);
-		fail(iBytesTooFar <= 0);
-		pCurr->tInfo.tLength -= (size_t)iBytesTooFar;
+		/* Reduce the part-unused2 block */
+		DBG_DEC(lBytesTooFar);
+		fail(lBytesTooFar <= 0);
+		pCurr->tInfo.lLength -= lBytesTooFar;
 		/* Move the integral number of garbage blocks */
 		pGarbageAnchor = pCurr->pNext;
 		pCurr->pNext = NULL;
@@ -467,8 +476,9 @@ vSplitBlockList(size_t tTextLen, size_t tFootnoteLen, size_t tUnused1Len,
 /* Free the garbage block list, this should never be needed */
 	pCurr = pGarbageAnchor;
 	while (pCurr != NULL) {
+		DBG_FIXME();
 		DBG_HEX(pCurr->tInfo.lFileOffset);
-		DBG_DEC(pCurr->tInfo.tLength);
+		DBG_DEC(pCurr->tInfo.lLength);
 		pNext = pCurr->pNext;
 		pCurr = xfree(pCurr);
 		pCurr = pNext;
@@ -476,94 +486,94 @@ vSplitBlockList(size_t tTextLen, size_t tFootnoteLen, size_t tUnused1Len,
 
 #if defined(DEBUG)
 	/* Check the number of bytes in the block lists */
-	tTotal = 0;
+	lTotal = 0;
 	for (pCurr = pTextAnchor; pCurr != NULL; pCurr = pCurr->pNext) {
 		NO_DBG_HEX(pCurr->tInfo.lFileOffset);
 		NO_DBG_HEX(pCurr->tInfo.lTextOffset);
-		NO_DBG_DEC(pCurr->tInfo.tLength);
-		fail(pCurr->tInfo.tLength == 0);
+		NO_DBG_DEC(pCurr->tInfo.lLength);
+		fail(pCurr->tInfo.lLength <= 0);
 		if (pCurr->tInfo.bUsesUnicode) {
-			fail(odd(pCurr->tInfo.tLength));
-			tTotal += pCurr->tInfo.tLength / 2;
+			fail(odd(pCurr->tInfo.lLength));
+			lTotal += pCurr->tInfo.lLength / 2;
 		} else {
-			tTotal += pCurr->tInfo.tLength;
+			lTotal += pCurr->tInfo.lLength;
 		}
 	}
-	DBG_DEC(tTotal);
-	if (tTotal != tTextLen) {
-		DBG_DEC(tTextLen);
+	DBG_DEC(lTotal);
+	if (lTotal != lTextLen) {
+		DBG_DEC(lTextLen);
 		werr(1, "Software error (Text)");
 	}
-	tTotal = 0;
+	lTotal = 0;
 	for (pCurr = pFootAnchor; pCurr != NULL; pCurr = pCurr->pNext) {
 		DBG_HEX(pCurr->tInfo.lFileOffset);
 		NO_DBG_HEX(pCurr->tInfo.lTextOffset);
-		DBG_DEC(pCurr->tInfo.tLength);
-		fail(pCurr->tInfo.tLength == 0);
+		DBG_DEC(pCurr->tInfo.lLength);
+		fail(pCurr->tInfo.lLength <= 0);
 		if (pCurr->tInfo.bUsesUnicode) {
-			fail(odd(pCurr->tInfo.tLength));
-			tTotal += pCurr->tInfo.tLength / 2;
+			fail(odd(pCurr->tInfo.lLength));
+			lTotal += pCurr->tInfo.lLength / 2;
 		} else {
-			tTotal += pCurr->tInfo.tLength;
+			lTotal += pCurr->tInfo.lLength;
 		}
 	}
-	DBG_DEC(tTotal);
-	if (tTotal != tFootnoteLen) {
-		DBG_DEC(tFootnoteLen);
+	DBG_DEC(lTotal);
+	if (lTotal != lFootnoteLen) {
+		DBG_DEC(lFootnoteLen);
 		werr(1, "Software error (Footnotes)");
 	}
-	tTotal = 0;
+	lTotal = 0;
 	for (pCurr = pUnused1Anchor; pCurr != NULL; pCurr = pCurr->pNext) {
 		DBG_HEX(pCurr->tInfo.lFileOffset);
 		NO_DBG_HEX(pCurr->tInfo.lTextOffset);
-		DBG_DEC(pCurr->tInfo.tLength);
-		fail(pCurr->tInfo.tLength == 0);
+		DBG_DEC(pCurr->tInfo.lLength);
+		fail(pCurr->tInfo.lLength <= 0);
 		if (pCurr->tInfo.bUsesUnicode) {
-			fail(odd(pCurr->tInfo.tLength));
-			tTotal += pCurr->tInfo.tLength / 2;
+			fail(odd(pCurr->tInfo.lLength));
+			lTotal += pCurr->tInfo.lLength / 2;
 		} else {
-			tTotal += pCurr->tInfo.tLength;
+			lTotal += pCurr->tInfo.lLength;
 		}
 	}
-	DBG_DEC(tTotal);
-	if (tTotal != tUnused1Len) {
-		DBG_DEC(tUnused1Len);
+	DBG_DEC(lTotal);
+	if (lTotal != lUnused1Len) {
+		DBG_DEC(lUnused1Len);
 		werr(1, "Software error (Unused1-list)");
 	}
-	tTotal = 0;
+	lTotal = 0;
 	for (pCurr = pEndAnchor; pCurr != NULL; pCurr = pCurr->pNext) {
 		DBG_HEX(pCurr->tInfo.lFileOffset);
 		NO_DBG_HEX(pCurr->tInfo.lTextOffset);
-		DBG_DEC(pCurr->tInfo.tLength);
-		fail(pCurr->tInfo.tLength == 0);
+		DBG_DEC(pCurr->tInfo.lLength);
+		fail(pCurr->tInfo.lLength <= 0);
 		if (pCurr->tInfo.bUsesUnicode) {
-			fail(odd(pCurr->tInfo.tLength));
-			tTotal += pCurr->tInfo.tLength / 2;
+			fail(odd(pCurr->tInfo.lLength));
+			lTotal += pCurr->tInfo.lLength / 2;
 		} else {
-			tTotal += pCurr->tInfo.tLength;
+			lTotal += pCurr->tInfo.lLength;
 		}
 	}
-	DBG_DEC(tTotal);
-	if (tTotal != tEndnoteLen) {
-		DBG_DEC(tEndnoteLen);
+	DBG_DEC(lTotal);
+	if (lTotal != lEndnoteLen) {
+		DBG_DEC(lEndnoteLen);
 		werr(1, "Software error (Endnotes)");
 	}
-	tTotal = 0;
+	lTotal = 0;
 	for (pCurr = pUnused2Anchor; pCurr != NULL; pCurr = pCurr->pNext) {
 		DBG_HEX(pCurr->tInfo.lFileOffset);
 		NO_DBG_HEX(pCurr->tInfo.lTextOffset);
-		DBG_DEC(pCurr->tInfo.tLength);
-		fail(pCurr->tInfo.tLength == 0);
+		DBG_DEC(pCurr->tInfo.lLength);
+		fail(pCurr->tInfo.lLength <= 0);
 		if (pCurr->tInfo.bUsesUnicode) {
-			fail(odd(pCurr->tInfo.tLength));
-			tTotal += pCurr->tInfo.tLength / 2;
+			fail(odd(pCurr->tInfo.lLength));
+			lTotal += pCurr->tInfo.lLength / 2;
 		} else {
-			tTotal += pCurr->tInfo.tLength;
+			lTotal += pCurr->tInfo.lLength;
 		}
 	}
-	DBG_DEC(tTotal);
-	if (tTotal != tUnused2Len) {
-		DBG_DEC(tUnused2Len);
+	DBG_DEC(lTotal);
+	if (lTotal != lUnused2Len) {
+		DBG_DEC(lUnused2Len);
 		werr(1, "Software error (Unused2-list)");
 	}
 #endif /* DEBUG */
@@ -587,14 +597,14 @@ vSplitBlockList(size_t tTextLen, size_t tFootnoteLen, size_t tUnused1Len,
 		     pCurr != NULL;
 		     pCurr = pCurr->pNext) {
 			if (pCurr->pNext != NULL &&
-			    pCurr->tInfo.tLength % BIG_BLOCK_SIZE != 0) {
+			    pCurr->tInfo.lLength % BIG_BLOCK_SIZE != 0) {
 				DBG_HEX(pCurr->tInfo.lFileOffset);
 				DBG_HEX(pCurr->tInfo.lTextOffset);
-				DBG_DEC(pCurr->tInfo.tLength);
-				pCurr->tInfo.tLength /= BIG_BLOCK_SIZE;
-				pCurr->tInfo.tLength++;
-				pCurr->tInfo.tLength *= BIG_BLOCK_SIZE;
-				DBG_DEC(pCurr->tInfo.tLength);
+				DBG_DEC(pCurr->tInfo.lLength);
+				pCurr->tInfo.lLength /= BIG_BLOCK_SIZE;
+				pCurr->tInfo.lLength++;
+				pCurr->tInfo.lLength *= BIG_BLOCK_SIZE;
+				DBG_DEC(pCurr->tInfo.lLength);
 			}
 		}
 	}
@@ -602,16 +612,16 @@ vSplitBlockList(size_t tTextLen, size_t tFootnoteLen, size_t tUnused1Len,
 
 #if defined(__riscos)
 /*
- * tGetDocumentLength - get the combined character length of the three lists
+ * lGetDocumentLength - get the combined character length of the three lists
  *
  * returns: The total number of characters
  */
-size_t
-tGetDocumentLength(void)
+long
+lGetDocumentLength(void)
 {
 	list_mem_type	*apAnchors[3];
 	list_mem_type	*pCurr;
-	size_t		tTotal;
+	long		lTotal;
 	int		iIndex;
 
 	DBG_MSG("uiGetDocumentLength");
@@ -620,22 +630,22 @@ tGetDocumentLength(void)
 	apAnchors[1] = pFootAnchor;
 	apAnchors[2] = pEndAnchor;
 
-	tTotal = 0;
+	lTotal = 0;
 	for (iIndex = 0; iIndex < 3; iIndex++) {
 		for (pCurr = apAnchors[iIndex];
 		     pCurr != NULL;
 		     pCurr = pCurr->pNext) {
-			fail(pCurr->tInfo.tLength == 0);
+			fail(pCurr->tInfo.lLength <= 0);
 			if (pCurr->tInfo.bUsesUnicode) {
-				fail(odd(pCurr->tInfo.tLength));
-				tTotal += pCurr->tInfo.tLength / 2;
+				fail(odd(pCurr->tInfo.lLength));
+				lTotal += pCurr->tInfo.lLength / 2;
 			} else {
-				tTotal += pCurr->tInfo.tLength;
+				lTotal += pCurr->tInfo.lLength;
 			}
 		}
 	}
-	DBG_DEC(tTotal);
-	return tTotal;
+	DBG_DEC(lTotal);
+	return lTotal;
 } /* end of tGetDocumentLength */
 #endif /* __riscos */
 
@@ -643,48 +653,56 @@ tGetDocumentLength(void)
  * usNextTextByte - get the next byte from the text block list
  */
 static unsigned short
-usNextTextByte(FILE *pFile, long *plOffset)
+usNextTextByte(FILE *pFile,
+	long *plFileOffset, long *plTextOffset, unsigned short *pusPropMod)
 {
+	static long	lBlockOffset = 0;
 	static size_t	tByteNext = 0;
-	static size_t	tBlockOffset = 0;
 	long	lReadOff;
 	size_t	tReadLen;
 
 	if (pTextBlockCurrent == NULL ||
 	    tByteNext >= sizeof(aucBlock) ||
-	    tBlockOffset + tByteNext >= pTextBlockCurrent->tInfo.tLength) {
+	    lBlockOffset + (long)tByteNext >= pTextBlockCurrent->tInfo.lLength) {
 		if (pTextBlockCurrent == NULL) {
 			/* First block, first part */
 			pTextBlockCurrent = pTextAnchor;
-			tBlockOffset = 0;
-		} else if (tBlockOffset + sizeof(aucBlock) <
-				pTextBlockCurrent->tInfo.tLength) {
+			lBlockOffset = 0;
+		} else if (lBlockOffset + (long)sizeof(aucBlock) <
+				pTextBlockCurrent->tInfo.lLength) {
 			/* Same block, next part */
-			tBlockOffset += sizeof(aucBlock);
+			lBlockOffset += (long)sizeof(aucBlock);
 		} else {
 			/* Next block, first part */
 			pTextBlockCurrent = pTextBlockCurrent->pNext;
-			tBlockOffset = 0;
+			lBlockOffset = 0;
 		}
 		if (pTextBlockCurrent == NULL) {
 			/* Past the last part of the last block */
 			return (unsigned short)EOF;
 		}
-		tReadLen = pTextBlockCurrent->tInfo.tLength - tBlockOffset;
+		tReadLen = (size_t)
+			(pTextBlockCurrent->tInfo.lLength - lBlockOffset);
 		if (tReadLen > sizeof(aucBlock)) {
 			tReadLen = sizeof(aucBlock);
 		}
 		lReadOff = pTextBlockCurrent->tInfo.lFileOffset +
-				(long)tBlockOffset;
+				lBlockOffset;
 		if (!bReadBytes(aucBlock, tReadLen, lReadOff, pFile)) {
 			return (unsigned short)EOF;
 		}
 		tByteNext = 0;
 	}
-	if (plOffset != NULL) {
-		*plOffset = pTextBlockCurrent->tInfo.lFileOffset +
-			(long)tBlockOffset +
-			(long)tByteNext;
+	if (plFileOffset != NULL) {
+		*plFileOffset = pTextBlockCurrent->tInfo.lFileOffset +
+			lBlockOffset + (long)tByteNext;
+	}
+	if (plTextOffset != NULL) {
+		*plTextOffset = pTextBlockCurrent->tInfo.lTextOffset +
+			lBlockOffset + (long)tByteNext;
+	}
+	if (pusPropMod != NULL) {
+		*pusPropMod = pTextBlockCurrent->tInfo.usPropMod;
 	}
 	return aucBlock[tByteNext++];
 } /* end of usNextTextByte */
@@ -693,48 +711,56 @@ usNextTextByte(FILE *pFile, long *plOffset)
  * usNextFootByte - get the next byte from the footnote block list
  */
 static unsigned short
-usNextFootByte(FILE *pFile, long *plOffset)
+usNextFootByte(FILE *pFile,
+	long *plFileOffset, long *plTextOffset, unsigned short *pusPropMod)
 {
+	static long	lBlockOffset = 0;
 	static size_t	tByteNext = 0;
-	static size_t	tBlockOffset = 0;
 	long	lReadOff;
 	size_t	tReadLen;
 
 	if (pFootBlockCurrent == NULL ||
 	    tByteNext >= sizeof(aucBlock) ||
-	    tBlockOffset + tByteNext >= pFootBlockCurrent->tInfo.tLength) {
+	    lBlockOffset + (long)tByteNext >= pFootBlockCurrent->tInfo.lLength) {
 		if (pFootBlockCurrent == NULL) {
 			/* First block, first part */
 			pFootBlockCurrent = pFootAnchor;
-			tBlockOffset = 0;
-		} else if (tBlockOffset + sizeof(aucBlock) <
-				pFootBlockCurrent->tInfo.tLength) {
+			lBlockOffset = 0;
+		} else if (lBlockOffset + (long)sizeof(aucBlock) <
+				pFootBlockCurrent->tInfo.lLength) {
 			/* Same block, next part */
-			tBlockOffset += sizeof(aucBlock);
+			lBlockOffset += (long)sizeof(aucBlock);
 		} else {
 			/* Next block, first part */
 			pFootBlockCurrent = pFootBlockCurrent->pNext;
-			tBlockOffset = 0;
+			lBlockOffset = 0;
 		}
 		if (pFootBlockCurrent == NULL) {
 			/* Past the last part of the last block */
 			return (unsigned short)EOF;
 		}
-		tReadLen = pFootBlockCurrent->tInfo.tLength - tBlockOffset;
+		tReadLen = (size_t)
+			(pFootBlockCurrent->tInfo.lLength - lBlockOffset);
 		if (tReadLen > sizeof(aucBlock)) {
 			tReadLen = sizeof(aucBlock);
 		}
 		lReadOff = pFootBlockCurrent->tInfo.lFileOffset +
-				(long)tBlockOffset;
+				lBlockOffset;
 		if (!bReadBytes(aucBlock, tReadLen, lReadOff, pFile)) {
 			return (unsigned short)EOF;
 		}
 		tByteNext = 0;
 	}
-	if (plOffset != NULL) {
-		*plOffset = pFootBlockCurrent->tInfo.lFileOffset +
-			(long)tBlockOffset +
-			(long)tByteNext;
+	if (plFileOffset != NULL) {
+		*plFileOffset = pFootBlockCurrent->tInfo.lFileOffset +
+			lBlockOffset + (long)tByteNext;
+	}
+	if (plTextOffset != NULL) {
+		*plTextOffset = pFootBlockCurrent->tInfo.lTextOffset +
+			lBlockOffset + (long)tByteNext;
+	}
+	if (pusPropMod != NULL) {
+		*pusPropMod = pFootBlockCurrent->tInfo.usPropMod;
 	}
 	return aucBlock[tByteNext++];
 } /* end of usNextFootByte */
@@ -743,48 +769,56 @@ usNextFootByte(FILE *pFile, long *plOffset)
  * usNextEndByte - get the next byte from the endnote block list
  */
 static unsigned short
-usNextEndByte(FILE *pFile, long *plOffset)
+usNextEndByte(FILE *pFile,
+	long *plFileOffset, long *plTextOffset, unsigned short *pusPropMod)
 {
+	static long	lBlockOffset = 0;
 	static size_t	tByteNext = 0;
-	static size_t	tBlockOffset = 0;
 	long	lReadOff;
 	size_t	tReadLen;
 
 	if (pEndBlockCurrent == NULL ||
 	    tByteNext >= sizeof(aucBlock) ||
-	    tBlockOffset + tByteNext >= pEndBlockCurrent->tInfo.tLength) {
+	    lBlockOffset + (long)tByteNext >= pEndBlockCurrent->tInfo.lLength) {
 		if (pEndBlockCurrent == NULL) {
 			/* First block, first part */
 			pEndBlockCurrent = pEndAnchor;
-			tBlockOffset = 0;
-		} else if (tBlockOffset + sizeof(aucBlock) <
-				pEndBlockCurrent->tInfo.tLength) {
+			lBlockOffset = 0;
+		} else if (lBlockOffset + (long)sizeof(aucBlock) <
+				pEndBlockCurrent->tInfo.lLength) {
 			/* Same block, next part */
-			tBlockOffset += sizeof(aucBlock);
+			lBlockOffset += sizeof(aucBlock);
 		} else {
 			/* Next block, first part */
 			pEndBlockCurrent = pEndBlockCurrent->pNext;
-			tBlockOffset = 0;
+			lBlockOffset = 0;
 		}
 		if (pEndBlockCurrent == NULL) {
 			/* Past the last part of the last block */
 			return (unsigned short)EOF;
 		}
-		tReadLen = pEndBlockCurrent->tInfo.tLength - tBlockOffset;
+		tReadLen = (size_t)
+			(pEndBlockCurrent->tInfo.lLength - lBlockOffset);
 		if (tReadLen > sizeof(aucBlock)) {
 			tReadLen = sizeof(aucBlock);
 		}
 		lReadOff = pEndBlockCurrent->tInfo.lFileOffset +
-				(long)tBlockOffset;
+				lBlockOffset;
 		if (!bReadBytes(aucBlock, tReadLen, lReadOff, pFile)) {
 			return (unsigned short)EOF;
 		}
 		tByteNext = 0;
 	}
-	if (plOffset != NULL) {
-		*plOffset = pEndBlockCurrent->tInfo.lFileOffset +
-			(long)tBlockOffset +
-			(long)tByteNext;
+	if (plFileOffset != NULL) {
+		*plFileOffset = pEndBlockCurrent->tInfo.lFileOffset +
+			lBlockOffset + (long)tByteNext;
+	}
+	if (plTextOffset != NULL) {
+		*plTextOffset = pEndBlockCurrent->tInfo.lTextOffset +
+			lBlockOffset + (long)tByteNext;
+	}
+	if (pusPropMod != NULL) {
+		*pusPropMod = pEndBlockCurrent->tInfo.usPropMod;
 	}
 	return aucBlock[tByteNext++];
 } /* end of usNextEndByte */
@@ -793,22 +827,24 @@ usNextEndByte(FILE *pFile, long *plOffset)
  * usNextTextChar - get the next character from the text block list
  */
 static unsigned short
-usNextTextChar(FILE *pFile, long *plOffset)
+usNextTextChar(FILE *pFile,
+	long *plFileOffset, long *plTextOffset, unsigned short *pusPropMod)
 {
 	unsigned short	usLSB, usMSB;
 
-	usLSB = usNextTextByte(pFile, plOffset);
+	usLSB = usNextTextByte(pFile, plFileOffset, plTextOffset, pusPropMod);
 	if (usLSB == (unsigned short)EOF) {
 		return (unsigned short)EOF;
 	}
 	if (pTextBlockCurrent->tInfo.bUsesUnicode) {
-		usMSB = usNextTextByte(pFile, NULL);
+		usMSB = usNextTextByte(pFile, NULL, NULL, NULL);
 	} else {
 		usMSB = 0x00;
 	}
 	if (usMSB == (unsigned short)EOF) {
 		DBG_MSG("usNextTextChar: Unexpected EOF");
-		DBG_HEX_C(plOffset != NULL, *plOffset);
+		DBG_HEX_C(plFileOffset != NULL, *plFileOffset);
+		DBG_HEX_C(plTextOffset != NULL, *plTextOffset);
 		return (unsigned short)EOF;
 	}
 	return (usMSB << 8) | usLSB;
@@ -818,22 +854,24 @@ usNextTextChar(FILE *pFile, long *plOffset)
  * usNextFootChar - get the next character from the footnote block list
  */
 static unsigned short
-usNextFootChar(FILE *pFile, long *plOffset)
+usNextFootChar(FILE *pFile,
+	long *plFileOffset, long *plTextOffset, unsigned short *pusPropMod)
 {
 	unsigned short	usLSB, usMSB;
 
-	usLSB = usNextFootByte(pFile, plOffset);
+	usLSB = usNextFootByte(pFile, plFileOffset, plTextOffset, pusPropMod);
 	if (usLSB == (unsigned short)EOF) {
 		return (unsigned short)EOF;
 	}
 	if (pFootBlockCurrent->tInfo.bUsesUnicode) {
-		usMSB = usNextFootByte(pFile, NULL);
+		usMSB = usNextFootByte(pFile, NULL, NULL, NULL);
 	} else {
 		usMSB = 0x00;
 	}
 	if (usMSB == (unsigned short)EOF) {
 		DBG_MSG("usNextFootChar: Unexpected EOF");
-		DBG_HEX_C(plOffset != NULL, *plOffset);
+		DBG_HEX_C(plFileOffset != NULL, *plFileOffset);
+		DBG_HEX_C(plTextOffset != NULL, *plTextOffset);
 		return (unsigned short)EOF;
 	}
 	return (usMSB << 8) | usLSB;
@@ -843,22 +881,24 @@ usNextFootChar(FILE *pFile, long *plOffset)
  * usNextEndChar - get the next character from the endnote block list
  */
 static unsigned short
-usNextEndChar(FILE *pFile, long *plOffset)
+usNextEndChar(FILE *pFile,
+	long *plFileOffset, long *plTextOffset, unsigned short *pusPropMod)
 {
 	unsigned short	usLSB, usMSB;
 
-	usLSB = usNextEndByte(pFile, plOffset);
+	usLSB = usNextEndByte(pFile, plFileOffset, plTextOffset, pusPropMod);
 	if (usLSB == (unsigned short)EOF) {
 		return (unsigned short)EOF;
 	}
 	if (pEndBlockCurrent->tInfo.bUsesUnicode) {
-		usMSB = usNextEndByte(pFile, NULL);
+		usMSB = usNextEndByte(pFile, NULL, NULL, NULL);
 	} else {
 		usMSB = 0x00;
 	}
 	if (usMSB == (unsigned short)EOF) {
 		DBG_MSG("usNextEndChar: Unexpected EOF");
-		DBG_HEX_C(plOffset != NULL, *plOffset);
+		DBG_HEX_C(plFileOffset != NULL, *plFileOffset);
+		DBG_HEX_C(plTextOffset != NULL, *plTextOffset);
 		return (unsigned short)EOF;
 	}
 	return (usMSB << 8) | usLSB;
@@ -868,27 +908,74 @@ usNextEndChar(FILE *pFile, long *plOffset)
  * usNextChar - get the next character from the given block list
  */
 unsigned short
-usNextChar(FILE *pFile, list_id_enum eListID, long *plOffset)
+usNextChar(FILE *pFile, list_id_enum eListID,
+	long *plFileOffset, long *plTextOffset, unsigned short *pusPropMod)
 {
 	fail(pFile == NULL);
 
 	switch (eListID) {
 	case text_list:
-		return usNextTextChar(pFile, plOffset);
+		return usNextTextChar(pFile,
+				plFileOffset, plTextOffset, pusPropMod);
 	case footnote_list:
-		return usNextFootChar(pFile, plOffset);
+		return usNextFootChar(pFile,
+				plFileOffset, plTextOffset, pusPropMod);
 	case endnote_list:
-		return usNextEndChar(pFile, plOffset);
+		return usNextEndChar(pFile,
+				plFileOffset, plTextOffset, pusPropMod);
 	default:
-		if (plOffset != NULL) {
-			*plOffset = -1;
+		if (plFileOffset != NULL) {
+			*plFileOffset = -1;
+		}
+		if (plTextOffset != NULL) {
+			*plTextOffset = -1;
+		}
+		if (pusPropMod != NULL) {
+			*pusPropMod = IGNORE_PROPMOD;
 		}
 		return (unsigned short)EOF;
 	}
 } /* end of usNextChar */
 
+#if 0
+/*
+ * Translate the start from the begin of the text to the block that contains
+ * this offset.
+ * Logical offset to block pointer.
+ *
+ * Returns: block pointer or NULL
+ */
+static text_block_type *
+pTextOffset2Block(long lTextOffset)
+{
+	list_mem_type	*apAnchors[3];
+	list_mem_type	*pCurr;
+	int		iIndex;
+
+	apAnchors[0] = pTextAnchor;
+	apAnchors[1] = pFootAnchor;
+	apAnchors[2] = pEndAnchor;
+
+	for (iIndex = 0; iIndex < 3; iIndex++) {
+		for (pCurr = apAnchors[iIndex];
+		     pCurr != NULL;
+		     pCurr = pCurr->pNext) {
+			if (lTextOffset >= pCurr->tInfo.lTextOffset &&
+			    lTextOffset < pCurr->tInfo.lTextOffset +
+			     pCurr->tInfo.lLength) {
+				/* The textoffset is in the current block */
+				return &pCurr->tInfo;
+			}
+		}
+	}
+	/* Passed beyond the end of the last list */
+	return NULL;
+} /* end of pTextOffset2Block */
+#endif
+
 /*
  * Translate the start from the begin of the text to an offset in the file.
+ * Logical to physical offset.
  *
  * Returns:	 -1: in case of error
  *		>=0: the computed file offset
@@ -896,89 +983,65 @@ usNextChar(FILE *pFile, list_id_enum eListID, long *plOffset)
 long
 lTextOffset2FileOffset(long lTextOffset)
 {
-	list_mem_type	*apAnchors[5];
+	list_mem_type	*apAnchors[3];
 	list_mem_type	*pCurr;
+	long		lBestGuess;
 	int		iIndex;
-	BOOL		bStartOfList;
 
 	apAnchors[0] = pTextAnchor;
 	apAnchors[1] = pFootAnchor;
-	apAnchors[2] = pUnused1Anchor;
-	apAnchors[3] = pEndAnchor;
-	apAnchors[4] = pUnused2Anchor;
+	apAnchors[2] = pEndAnchor;
 
-	bStartOfList = FALSE;
+	lBestGuess = -1;	/* Best guess is "fileoffset not found" */
 
-	for (iIndex = 0; iIndex < 5; iIndex++) {
-		pCurr = apAnchors[iIndex];
-		while (pCurr != NULL) {
-			if (bStartOfList) {
-				NO_DBG_DEC(iIndex);
-				NO_DBG_HEX(pCurr->tInfo.lFileOffset);
-				if (iIndex >= 4) {
-					/* Past the last used byte */
-					return -1;
-				}
-				return pCurr->tInfo.lFileOffset;
+	for (iIndex = 0; iIndex < 3; iIndex++) {
+		for (pCurr = apAnchors[iIndex];
+		     pCurr != NULL;
+		     pCurr = pCurr->pNext) {
+			if (lTextOffset == pCurr->tInfo.lTextOffset +
+			     pCurr->tInfo.lLength &&
+			    pCurr->pNext != NULL) {
+				/*
+				 * The textoffset is one beyond this block,
+				 * so we guess it's the first byte of the next
+				 * block (if there is a next block)
+				 */
+				lBestGuess = pCurr->pNext->tInfo.lFileOffset;
 			}
+
 			if (lTextOffset < pCurr->tInfo.lTextOffset ||
 			    lTextOffset >= pCurr->tInfo.lTextOffset +
-			     (long)pCurr->tInfo.tLength) {
-				pCurr = pCurr->pNext;
+			     pCurr->tInfo.lLength) {
+				/* The textoffset is not in this block */
 				continue;
 			}
-			switch (iIndex) {
-			case 0:
-			case 1:
-			case 3:
+
 			/* The textoffset is in the current block */
-				return pCurr->tInfo.lFileOffset +
-					lTextOffset -
-					pCurr->tInfo.lTextOffset;
-			case 2:
-			/* Use the start of the next non-empty list */
-				bStartOfList = TRUE;
-				break;
-			case 4:
-			/* In Unused2, means after the last used byte */
-				return -1;
-			default:
-			/* This should not happen */
-				return -1;
-			}
-			/* To the start of the next list */
-			break;
+			return pCurr->tInfo.lFileOffset +
+				lTextOffset - pCurr->tInfo.lTextOffset;
 		}
 	}
 	/* Passed beyond the end of the last list */
 	NO_DBG_HEX(lTextOffset);
-	return -1;
+	NO_DBG_HEX(lBestGuess);
+	return lBestGuess;
 } /* end of lTextOffset2FileOffset */
 
+#if 0
 /*
- * Get the sequence number beloning to the given file offset
+ * Get the properties modifier for the given logical offset
  *
- * Returns the sequence number
+ * Returns: the properties modifier or IGNORE_PROPMOD
  */
-long
-lGetSeqNumber(long lFileOffset)
+unsigned short
+pTextOffset2PropMod(long lTextOffset)
 {
-	list_mem_type	*pCurr;
-	long		lSeq;
+	text_block_type	*pTextBlock;
 
-	if (lFileOffset < 0) {
-		return -1;
+	pTextBlock = pTextOffset2Block(lTextOffset);
+	if (pTextBlock == NULL) {
+		return IGNORE_PROPMOD;
 	}
-
-	lSeq = 0;
-	for (pCurr = pTextAnchor; pCurr != NULL; pCurr = pCurr->pNext) {
-		if (lFileOffset >= pCurr->tInfo.lFileOffset &&
-		    lFileOffset < pCurr->tInfo.lFileOffset +
-		     (long)pCurr->tInfo.tLength) {
-			/* The file offset is within the current textblock */
-			return lSeq + lFileOffset - pCurr->tInfo.lFileOffset;
-		}
-		lSeq += (long)pCurr->tInfo.tLength;
-	}
-	return -1;
-} /* end of lGetSeqNumber */
+	return pTextBlock->usPropMod;
+} /* end of pTextOffset2PropMod */
+#endif
