@@ -85,7 +85,8 @@ bFontEqual(const char *szWord, const char *szTable, int iBytesPerChar)
 	for (pcTmp1 = szWord, pcTmp2 = szTable;
 	     *pcTmp1 != '\0';
 	     pcTmp1 += iBytesPerChar, pcTmp2++) {
-		if (iToUpper(*pcTmp1) != iToUpper(*pcTmp2)) {
+		if (ulToUpper((unsigned long)*pcTmp1) !=
+		    ulToUpper((unsigned long)*pcTmp2)) {
 			return FALSE;
 		}
 	}
@@ -300,22 +301,23 @@ bReadFontFile(FILE *pFontTableFile, char *szWordFont,
  * vCreate6FontTable - create a font table from Word 6/7
  */
 void
-vCreate6FontTable(FILE *pFile, int iStartBlock,
-	const int *aiBBD, size_t tBBDLen,
+vCreate6FontTable(FILE *pFile, long lStartBlock,
+	const long *alBBD, size_t tBBDLen,
 	const unsigned char *aucHeader)
 {
 	FILE	*pFontTableFile;
 	font_table_type	*pTmp;
 	char	*szFont, *szAltFont;
 	unsigned char	*aucBuffer;
-	size_t	tBeginFontInfo, tFontInfoLen;
+	long	lBeginFontInfo;
+	size_t	tFontInfoLen;
 	int	iPos, iRecLen, iOffsetAltName;
 	int	iBold, iItalic, iSpecial;
 	char	szWordFont[81], szOurFont[81];
 
 	fail(pFile == NULL || aucHeader == NULL);
-	fail(iStartBlock < 0);
-	fail(aiBBD == NULL);
+	fail(lStartBlock < 0);
+	fail(alBBD == NULL);
 
 	pFontTableFile = pOpenFontTableFile();
 	if (pFontTableFile == NULL) {
@@ -323,16 +325,16 @@ vCreate6FontTable(FILE *pFile, int iStartBlock,
 		return;
 	}
 
-	tBeginFontInfo = (size_t)ulGetLong(0xd0, aucHeader);
-	DBG_HEX(tBeginFontInfo);
+	lBeginFontInfo = (long)ulGetLong(0xd0, aucHeader);
+	DBG_HEX(lBeginFontInfo);
 	tFontInfoLen = (size_t)ulGetLong(0xd4, aucHeader);
 	DBG_DEC(tFontInfoLen);
 	fail(tFontInfoLen < 9);
 
 	aucBuffer = xmalloc(tFontInfoLen);
-	if (!bReadBuffer(pFile, iStartBlock,
-			aiBBD, tBBDLen, BIG_BLOCK_SIZE,
-			aucBuffer, tBeginFontInfo, tFontInfoLen)) {
+	if (!bReadBuffer(pFile, lStartBlock,
+			alBBD, tBBDLen, BIG_BLOCK_SIZE,
+			aucBuffer, lBeginFontInfo, tFontInfoLen)) {
 		aucBuffer = xfree(aucBuffer);
 		(void)fclose(pFontTableFile);
 		return;
@@ -404,24 +406,23 @@ vCreate6FontTable(FILE *pFile, int iStartBlock,
  */
 void
 vCreate8FontTable(FILE *pFile, const pps_info_type *pPPS,
-	const int *aiBBD, size_t tBBDLen, const int *aiSBD, size_t tSBDLen,
+	const long *alBBD, size_t tBBDLen, const long *alSBD, size_t tSBDLen,
 	const unsigned char *aucHeader)
 {
 	FILE	*pFontTableFile;
 	font_table_type	*pTmp;
-	const int	*aiBlockDepot;
+	const long	*alBlockDepot;
 	char		*szFont, *szAltFont;
 	unsigned char	*aucBuffer;
-	long	lTableSize;
-	size_t	tBeginFontInfo, tFontInfoLen, tBlockDepotLen, tBlockSize;
-	int	iTableStartBlock;
+	long	lTableSize, lTableStartBlock, lBeginFontInfo;
+	size_t	tFontInfoLen, tBlockDepotLen, tBlockSize;
 	int	iPos, iRecLen, iOffsetAltName;
 	int	iBold, iItalic, iSpecial;
 	unsigned short	usDocStatus;
 	char	szWordFont[81], szOurFont[81];
 
 	fail(pFile == NULL || pPPS == NULL || aucHeader == NULL);
-	fail(aiBBD == NULL || aiSBD == NULL);
+	fail(alBBD == NULL || alSBD == NULL);
 
 	pFontTableFile = pOpenFontTableFile();
 	if (pFontTableFile == NULL) {
@@ -429,8 +430,8 @@ vCreate8FontTable(FILE *pFile, const pps_info_type *pPPS,
 		return;
 	}
 
-	tBeginFontInfo = (size_t)ulGetLong(0x112, aucHeader);
-	DBG_HEX(tBeginFontInfo);
+	lBeginFontInfo = (long)ulGetLong(0x112, aucHeader);
+	DBG_HEX(lBeginFontInfo);
 	tFontInfoLen = (size_t)ulGetLong(0x116, aucHeader);
 	DBG_DEC(tFontInfoLen);
 	fail(tFontInfoLen < 46);
@@ -438,34 +439,34 @@ vCreate8FontTable(FILE *pFile, const pps_info_type *pPPS,
 	/* Use 0Table or 1Table? */
 	usDocStatus = usGetWord(0x0a, aucHeader);
 	if (usDocStatus & BIT(9)) {
-		iTableStartBlock = pPPS->t1Table.iSb;
+		lTableStartBlock = pPPS->t1Table.lSb;
 		lTableSize = pPPS->t1Table.lSize;
 	} else {
-		iTableStartBlock = pPPS->t0Table.iSb;
+		lTableStartBlock = pPPS->t0Table.lSb;
 		lTableSize = pPPS->t0Table.lSize;
 	}
-	DBG_DEC(iTableStartBlock);
-	if (iTableStartBlock < 0) {
-		DBG_DEC(iTableStartBlock);
+	DBG_DEC(lTableStartBlock);
+	if (lTableStartBlock < 0) {
+		DBG_DEC(lTableStartBlock);
 		DBG_MSG("No fontname table");
 		return;
 	}
 	DBG_HEX(lTableSize);
 	if (lTableSize < MIN_SIZE_FOR_BBD_USE) {
 	  	/* Use the Small Block Depot */
-		aiBlockDepot = aiSBD;
+		alBlockDepot = alSBD;
 		tBlockDepotLen = tSBDLen;
 		tBlockSize = SMALL_BLOCK_SIZE;
 	} else {
 	  	/* Use the Big Block Depot */
-		aiBlockDepot = aiBBD;
+		alBlockDepot = alBBD;
 		tBlockDepotLen = tBBDLen;
 		tBlockSize = BIG_BLOCK_SIZE;
 	}
 	aucBuffer = xmalloc(tFontInfoLen);
-	if (!bReadBuffer(pFile, iTableStartBlock,
-			aiBlockDepot, tBlockDepotLen, tBlockSize,
-			aucBuffer, tBeginFontInfo, tFontInfoLen)) {
+	if (!bReadBuffer(pFile, lTableStartBlock,
+			alBlockDepot, tBlockDepotLen, tBlockSize,
+			aucBuffer, lBeginFontInfo, tFontInfoLen)) {
 		aucBuffer = xfree(aucBuffer);
 		(void)fclose(pFontTableFile);
 		return;
