@@ -1,6 +1,6 @@
 /*
  * options.c
- * Copyright (C) 1998-2000 A.J. van Os; Released under GPL
+ * Copyright (C) 1998-2001 A.J. van Os; Released under GPL
  *
  * Description:
  * Read and write the options
@@ -102,6 +102,7 @@ iReadOptions(int argc, char **argv)
 	char	*pcChar, *szTmp;
 	int	iChar;
 	BOOL	bPostScript, bFound;
+	char	szLeafname[32+1];
 	char	szMappingFile[PATH_MAX+1];
 #endif /* __riscos */
 	int	iTmp;
@@ -178,16 +179,14 @@ iReadOptions(int argc, char **argv)
 			DBG_DEC(tOptionsCurr.iParagraphBreak);
 		}
 	}
-/* Command line */
-	szHome = szGetHomeDirectory();
-	if (strlen(szHome) >= sizeof(szMappingFile) -
-			sizeof(FILE_SEPARATOR) -
-			sizeof(MAPPING_FILE_DEFAULT_1)) {
-		werr(0, "The name of your home directory is too long");
-		return -1;
+	if (is_locale_utf8()) {
+		tOptionsCurr.eEncoding = encoding_utf8;
+		strcpy(szLeafname, MAPPING_FILE_DEFAULT_8);
+	} else {
+		tOptionsCurr.eEncoding = encoding_iso_8859_1;
+		strcpy(szLeafname, MAPPING_FILE_DEFAULT_1);
 	}
-	sprintf(szMappingFile, "%s" FILE_SEPARATOR MAPPING_FILE_DEFAULT_1,
-		szHome);
+/* Command line */
 	bPostScript = FALSE;
 	while ((iChar = getopt(argc, argv, "LX:hi:m:p:stw:")) != -1) {
 		switch (iChar) {
@@ -199,20 +198,17 @@ iReadOptions(int argc, char **argv)
 			switch (iTmp) {
 			case 1:
 				tOptionsCurr.eEncoding = encoding_iso_8859_1;
-				sprintf(szMappingFile,
-				"%s" FILE_SEPARATOR MAPPING_FILE_DEFAULT_1,
-					szHome);
+				strcpy(szLeafname, MAPPING_FILE_DEFAULT_1);
 				break;
 			case 2:
 				tOptionsCurr.eEncoding = encoding_iso_8859_2;
-				sprintf(szMappingFile,
-				"%s" FILE_SEPARATOR MAPPING_FILE_DEFAULT_2,
-					szHome);
+				strcpy(szLeafname, MAPPING_FILE_DEFAULT_2);
 				break;
 			default:
+				/* Ignore */
 				break;
 			}
-			DBG_MSG(szMappingFile);
+			DBG_MSG(szLeafname);
 			break;
 		case 'h':
 			return 0;
@@ -241,21 +237,17 @@ iReadOptions(int argc, char **argv)
 			DBG_DEC(tOptionsCurr.eImageLevel);
 			break;
 		case 'm':
-			if (optarg[0] == '/') {
-				if (strlen(optarg) < sizeof(szMappingFile)) {
-					strcpy(szMappingFile, optarg);
-				}
+			strncpy(szLeafname, optarg, sizeof(szLeafname) - 1);
+			szLeafname[sizeof(szLeafname) - 1] = '\0';
+			DBG_MSG(szLeafname);
+			if (STREQ(szLeafname, MAPPING_FILE_DEFAULT_8)) {
+				tOptionsCurr.eEncoding = encoding_utf8;
+			} else if (STREQ(szLeafname, MAPPING_FILE_DEFAULT_2)) {
+				tOptionsCurr.eEncoding = encoding_iso_8859_2;
 			} else {
-				if (strlen(szHome) + strlen(optarg) <
-						sizeof(szMappingFile) -
-						sizeof(ANTIWORD_DIR) -
-						2 * sizeof(FILE_SEPARATOR)) {
-					sprintf(szMappingFile,
-			"%s" FILE_SEPARATOR ANTIWORD_DIR FILE_SEPARATOR "%s",
-						szHome, optarg);
-				}
+				tOptionsCurr.eEncoding = encoding_iso_8859_1;
 			}
-			DBG_MSG(szMappingFile);
+			DBG_DEC(tOptionsCurr.eEncoding);
 			break;
 		case 'p':
 			bPostScript = TRUE;
@@ -320,9 +312,38 @@ iReadOptions(int argc, char **argv)
 				PS_LEFT_MARGIN + PS_RIGHT_MARGIN));
 		DBG_DEC(tOptionsCurr.iParagraphBreak);
 	}
-	if (bReadCharacterMappingTable(szMappingFile)) {
-		return optind;
+	/* Try the local version of the mapping file */
+	szHome = szGetHomeDirectory();
+	if (strlen(szHome) + strlen(szLeafname) <
+	    sizeof(szMappingFile) -
+	    sizeof(ANTIWORD_DIR) -
+	    2 * sizeof(FILE_SEPARATOR)) {
+		sprintf(szMappingFile,
+			"%s" FILE_SEPARATOR ANTIWORD_DIR FILE_SEPARATOR "%s",
+			szHome, szLeafname);
+		DBG_MSG(szMappingFile);
+		if (bReadCharacterMappingTable(szMappingFile)) {
+			return optind;
+		}
+	} else {
+		werr(0, "Local mappingfilename too long, ignored");
 	}
+	/* Try the global version of the mapping file */
+	if (strlen(szLeafname) <
+	    sizeof(szMappingFile) -
+	    sizeof(GLOBAL_ANTIWORD_DIR) -
+	    sizeof(FILE_SEPARATOR)) {
+		sprintf(szMappingFile,
+			GLOBAL_ANTIWORD_DIR FILE_SEPARATOR "%s",
+			szLeafname);
+		DBG_MSG(szMappingFile);
+		if (bReadCharacterMappingTable(szMappingFile)) {
+			return optind;
+		}
+	} else {
+		werr(0, "Global mappingfilename too long, ignored");
+	}
+	werr(0, "I can't open your mapping file (%s)", szLeafname);
 	return -1;
 #endif /* __riscos */
 } /* end of iReadOptions */
