@@ -1,6 +1,6 @@
 /*
  * postscript.c
- * Copyright (C) 1999,2000 A.J. van Os; Released under GPL
+ * Copyright (C) 1999-2001 A.J. van Os; Released under GPL
  *
  * Description:
  * Functions to deal with the PostScript format
@@ -31,8 +31,6 @@ static image_level_enum	eImageLevel = level_default;
 static BOOL		bUseLandscape = FALSE;
 /* The height of a PostScript page (in DrawUnits) */
 static long		lPageHeight = LONG_MAX;
-/* The net width of a PostScript page (in points) */
-static double		dNetPageWidth = (double)ULONG_MAX;
 /* Current time for a PS header */
 static const char	*szCreationDate = NULL;
 /* Current creator for a PS header */
@@ -210,7 +208,7 @@ vMove2NextPage(diagram_type *pDiag)
 /*
  * vMoveToPS - move to the given X,Y coordinates (Postscript)
  *
- * Move the current position of the given diagram to its X,Y coordianates,
+ * Move the current position of the given diagram to its X,Y coordinates,
  * start on a new page if needed
  */
 static void
@@ -235,7 +233,7 @@ vMoveToPS(diagram_type *pDiag, long lLastVerticalMovement)
 } /* end of vMoveToPS */
 
 /*
- * vPrologue - perform the PostScript initialisation
+ * vPrologue - perform the PostScript initialization
  */
 static void
 vPrologue(FILE *pOutFile, const char *szTask, const char *szFilename)
@@ -250,10 +248,6 @@ vPrologue(FILE *pOutFile, const char *szTask, const char *szFilename)
 	vGetOptions(&tOptions);
 	lPageHeight = lPoints2DrawUnits(tOptions.iPageHeight);
 	DBG_DEC(lPageHeight);
-	dNetPageWidth = dDrawUnits2Points(
-		lPoints2DrawUnits(tOptions.iPageWidth) -
-		(PS_LEFT_MARGIN + PS_RIGHT_MARGIN));
-	DBG_FLT(dNetPageWidth);
 	bUsePostScript = tOptions.bUseOutlineFonts;
 	bUseLandscape = tOptions.bUseLandscape;
 	eEncoding = tOptions.eEncoding;
@@ -356,14 +350,12 @@ vPrintPalette(FILE *pOutFile, const imagedata_type *pImg)
 } /* end of vPrintPalette */
 
 /*
- * vImagePrologue - perform the Encapsulated PostScript initialisation
+ * vImagePrologue - perform the Encapsulated PostScript initialization
  */
 void
 vImagePrologue(diagram_type *pDiag, const imagedata_type *pImg)
 {
 	FILE	*pOutFile;
-	double	dHor, dVer;
-	int	iHorSize, iVerSize;
 
 	fail(pDiag == NULL);
 	fail(pDiag->pOutFile == NULL);
@@ -373,29 +365,20 @@ vImagePrologue(diagram_type *pDiag, const imagedata_type *pImg)
 		return;
 	}
 
+	if (pImg->iVerSizeScaled <= 0 || pImg->iHorSizeScaled <= 0) {
+		return;
+	}
+
 	fail(szCreationDate == NULL);
 	fail(szCreator == NULL);
 	fail(eImageLevel == level_no_images);
 
 	iImageCount++;
 
-	/* Scaling */
-	dHor = pImg->iHorizontalSize * pImg->dHorizontalScalingFactor;
-	dVer = pImg->iVerticalSize * pImg->dVerticalScalingFactor;
-
-	/* Make sure the image fits on the paper */
-	if (dHor < dNetPageWidth) {
-		iHorSize = (int)(dHor + 0.5);
-		iVerSize = (int)(dVer + 0.5);
-	} else {
-		iHorSize = (int)(dNetPageWidth + 0.5);
-		iVerSize = (int)(dVer * dNetPageWidth / dHor + 0.5);
-		DBG_DEC(iVerSize);
-	}
-
 	DBG_DEC_C(pDiag->lXleft != 0, pDiag->lXleft);
-	pDiag->lYtop -= lPoints2DrawUnits(iVerSize);
-	vMoveToPS(pDiag, lPoints2DrawUnits(iVerSize));
+
+	pDiag->lYtop -= lPoints2DrawUnits(pImg->iVerSizeScaled);
+	vMoveToPS(pDiag, lPoints2DrawUnits(pImg->iVerSizeScaled));
 
 	pOutFile = pDiag->pOutFile;
 
@@ -405,7 +388,8 @@ vImagePrologue(diagram_type *pDiag, const imagedata_type *pImg)
 	fprintf(pOutFile, "%%%%Creator: %s %s\n", szCreator, VERSIONSTRING);
 	fprintf(pOutFile, "%%%%Title: Image %03d\n", iImageCount);
 	fprintf(pOutFile, "%%%%CreationDate: %s", szCreationDate);
-	fprintf(pOutFile, "%%%%BoundingBox: 0 0 %d %d\n", iHorSize, iVerSize);
+	fprintf(pOutFile, "%%%%BoundingBox: 0 0 %d %d\n",
+				pImg->iHorSizeScaled, pImg->iVerSizeScaled);
 	fprintf(pOutFile, "%%%%DocumentData: Clean7Bit\n");
 	fprintf(pOutFile, "%%%%LanguageLevel: 2\n");
 	fprintf(pOutFile, "%%%%EndComments\n");
@@ -486,7 +470,7 @@ vImagePrologue(diagram_type *pDiag, const imagedata_type *pImg)
 			dDrawUnits2Points(pDiag->lYtop));
 
 	fprintf(pOutFile, "%d %d scale\n",
-			iHorSize, iVerSize);
+				pImg->iHorSizeScaled, pImg->iVerSizeScaled);
 
 	fprintf(pOutFile, "{ <<\n");
 	fprintf(pOutFile, "\t/ImageType 1\n");
@@ -598,14 +582,12 @@ vImageEpilogue(diagram_type *pDiag)
 /*
  * bAddDummyImage - add a dummy image
  *
- * return TRUE when sucessful, otherwise FALSE
+ * return TRUE when successful, otherwise FALSE
  */
 BOOL
 bAddDummyImage(diagram_type *pDiag, const imagedata_type *pImg)
 {
 	FILE	*pOutFile;
-	double	dHor, dVer;
-	int	iHorSize, iVerSize;
 
 	fail(pDiag == NULL);
 	fail(pDiag->pOutFile == NULL);
@@ -615,29 +597,16 @@ bAddDummyImage(diagram_type *pDiag, const imagedata_type *pImg)
 		return FALSE;
 	}
 
-	if (pImg->iVerticalSize <= 0 || pImg->iHorizontalSize <= 0) {
+	if (pImg->iVerSizeScaled <= 0 || pImg->iHorSizeScaled <= 0) {
 		return FALSE;
 	}
 
 	iImageCount++;
 
-	/* Scaling */
-	dHor = pImg->iHorizontalSize * pImg->dHorizontalScalingFactor;
-	dVer = pImg->iVerticalSize * pImg->dVerticalScalingFactor;
-
-	/* Make sure the dummy fits on the paper */
-	if (dHor < dNetPageWidth) {
-		iHorSize = (int)(dHor + 0.5);
-		iVerSize = (int)(dVer + 0.5);
-	} else {
-		iHorSize = (int)(dNetPageWidth + 0.5);
-		iVerSize = (int)(dVer * dNetPageWidth / dHor + 0.5);
-		DBG_DEC(iVerSize);
-	}
-
 	DBG_DEC_C(pDiag->lXleft != 0, pDiag->lXleft);
-	pDiag->lYtop -= lPoints2DrawUnits(iVerSize);
-	vMoveToPS(pDiag, lPoints2DrawUnits(iVerSize));
+
+	pDiag->lYtop -= lPoints2DrawUnits(pImg->iVerSizeScaled);
+	vMoveToPS(pDiag, lPoints2DrawUnits(pImg->iVerSizeScaled));
 
 	pOutFile = pDiag->pOutFile;
 
@@ -648,9 +617,9 @@ bAddDummyImage(diagram_type *pDiag, const imagedata_type *pImg)
 			dDrawUnits2Points(pDiag->lYtop));
 	fprintf(pOutFile, "\t1.0 setlinewidth\n");
 	fprintf(pOutFile, "\t0.3 setgray\n");
-	fprintf(pOutFile, "\t0 %d rlineto\n", iVerSize);
-	fprintf(pOutFile, "\t%d 0 rlineto\n", iHorSize);
-	fprintf(pOutFile, "\t0 %d rlineto\n", -iVerSize);
+	fprintf(pOutFile, "\t0 %d rlineto\n", pImg->iVerSizeScaled);
+	fprintf(pOutFile, "\t%d 0 rlineto\n", pImg->iHorSizeScaled);
+	fprintf(pOutFile, "\t0 %d rlineto\n", -pImg->iVerSizeScaled);
 	fprintf(pOutFile, "\tclosepath\n");
 	fprintf(pOutFile, "\tstroke\n");
 	fprintf(pOutFile, "grestore\n");
@@ -675,7 +644,7 @@ pCreateDiagram(const char *szTask, const char *szFilename)
 
 	/* Get the necessary memory */
 	pDiag = xmalloc(sizeof(diagram_type));
-	/* Initialisation */
+	/* Initialization */
 	pDiag->pOutFile = stdout;
 	vPrologue(pDiag->pOutFile, szTask, szFilename);
 	iPageCount = 0;
@@ -853,7 +822,7 @@ vSetColor(FILE *pFile, int iColor)
 /*
  * vMoveToASCII - move to the given X,Y coordinates (ASCII)
  *
- * Move the current position of the given diagram to its X,Y coordianates,
+ * Move the current position of the given diagram to its X,Y coordinates,
  * start on a new page if needed
  */
 static void
@@ -883,14 +852,14 @@ vMove2NextLine(diagram_type *pDiag, draw_fontref tFontRef, int iFontsize)
 	fail(pDiag->pOutFile == NULL);
 	fail(iFontsize < MIN_FONT_SIZE || iFontsize > MAX_FONT_SIZE);
 
-	pDiag->lYtop -= lWord2DrawUnits20(iFontsize);
+	pDiag->lYtop -= lComputeLeading(iFontsize);
 	if (!bUsePostScript) {
 		(void)fprintf(pDiag->pOutFile, "\n");
 	}
 } /* end of vMove2NextLine */
 
 /*
- * vSubstring2Diagram - put a substring into a diagram
+ * vSubstring2Diagram - put a sub string into a diagram
  */
 void
 vSubstring2Diagram(diagram_type *pDiag,
@@ -903,7 +872,8 @@ vSubstring2Diagram(diagram_type *pDiag,
 	fail(pDiag == NULL || szString == NULL);
 	fail(pDiag->pOutFile == NULL);
 	fail(pDiag->lXleft < 0);
-	fail(iStringLength != (int)strlen(szString));
+	fail(iStringLength < 0);
+	fail((size_t)iStringLength != strlen(szString));
 	fail(iFontsize < MIN_FONT_SIZE || iFontsize > MAX_FONT_SIZE);
 	fail(iMaxFontsize < MIN_FONT_SIZE || iMaxFontsize > MAX_FONT_SIZE);
 	fail(iFontsize > iMaxFontsize);
@@ -927,7 +897,7 @@ vSubstring2Diagram(diagram_type *pDiag,
 			vSetColor(pDiag->pOutFile, iColor);
 			iColorCurr = iColor;
 		}
-		vMoveToPS(pDiag, lWord2DrawUnits20(iMaxFontsize));
+		vMoveToPS(pDiag, lComputeLeading(iMaxFontsize));
 		vPrintPS(pDiag->pOutFile, szString, iStringLength, ucFontstyle);
 	} else {
 		vMoveToASCII(pDiag);
@@ -954,7 +924,7 @@ vEndOfParagraph2Diagram(diagram_type *pDiag,
 	/* Empty line */
 	if (bUsePostScript) {
 		pDiag->lXleft = 0;
-		pDiag->lYtop -= lWord2DrawUnits20(iFontsize) / 3;
+		pDiag->lYtop -= lComputeLeading(iFontsize) / 3;
 	} else {
 		vMove2NextLine(pDiag, tFontRef, iFontsize);
 	}
