@@ -1,6 +1,6 @@
 /*
  * word2text.c
- * Copyright (C) 1998-2000 A.J. van Os; Released under GPL
+ * Copyright (C) 1998-2001 A.J. van Os; Released under GPL
  *
  * Description:
  * MS Word to text functions
@@ -36,18 +36,22 @@
 
 #if defined(__riscos)
 /* Length of the document in characters */
-static size_t	tDocumentLength;
+static long	lDocumentLength;
 /* Number of characters processed so far */
-static size_t	tCharCounter;
+static long	lCharCounter;
 static int	iCurrPct, iPrevPct;
 #endif /* __riscos */
-/* Special treatment for files from the Apple Macintosh */
-static BOOL	bMacFile = FALSE;
+/* The document is in the format belonging to this version of Word */
+static int	iWordVersion = -1;
+/* Special treatment for files from Word 6 on an Apple Macintosh */
+static BOOL	bWord6MacFile = FALSE;
+/* All the (command line) options */
+static options_type	tOptions;
 /* Needed for reading a complete table row */
-static row_block_type	tRowInfo;
-static BOOL	bRowInfo = FALSE;
+static const row_block_type	*pRowInfo = NULL;
 static BOOL	bStartRow = FALSE;
-static BOOL	bEndRow = FALSE;
+static BOOL	bEndRowNorm = FALSE;
+static BOOL	bEndRowFast = FALSE;
 static BOOL	bIsTableRow = FALSE;
 /* Needed for finding the start of a style */
 static const style_block_type	*pStyleInfo = NULL;
@@ -66,8 +70,8 @@ static void
 vUpdateCounters(void)
 {
 #if defined(__riscos)
-	tCharCounter++;
-	iCurrPct = (int)((tCharCounter * 100) / tDocumentLength);
+	lCharCounter++;
+	iCurrPct = (int)((lCharCounter * 100) / lDocumentLength);
 	if (iCurrPct != iPrevPct) {
 		visdelay_percent(iCurrPct);
 		iPrevPct = iCurrPct;
@@ -76,7 +80,7 @@ vUpdateCounters(void)
 } /* end of vUpdateCounters */
 
 /*
- * bOutputContainsText - see if the output contains more than white-space
+ * bOutputContainsText - see if the output contains more than white space
  */
 static BOOL
 bOutputContainsText(output_type *pAnchor)
@@ -89,12 +93,11 @@ bOutputContainsText(output_type *pAnchor)
 	for (pTmp = pAnchor; pTmp != NULL; pTmp = pTmp->pNext) {
 		fail(pTmp->lStringWidth < 0);
 		for (iIndex = 0; iIndex < pTmp->iNextFree; iIndex++) {
-			if (isspace(pTmp->szStorage[iIndex])) {
+			if (isspace((int)pTmp->szStorage[iIndex])) {
 				continue;
 			}
 #if defined(DEBUG)
-			if ((unsigned char)pTmp->szStorage[iIndex] ==
-			    FILLER_CHAR) {
+			if (pTmp->szStorage[iIndex] == FILLER_CHAR) {
 				continue;
 			}
 #endif /* DEBUG */
@@ -122,14 +125,14 @@ lTotalStringWidth(output_type *pAnchor)
 } /* end of lTotalStringWidth */
 
 /*
- * vStoreCharacter - store one character
+ * vStoreByte - store one byte
  */
 static void
-vStoreCharacter(int iChar, output_type *pOutput)
+vStoreByte(unsigned char ucChar, output_type *pOutput)
 {
 	fail(pOutput == NULL);
 
-	if (iChar == 0) {
+	if (ucChar == 0) {
 		pOutput->szStorage[pOutput->iNextFree] = '\0';
 		return;
 	}
@@ -138,14 +141,51 @@ vStoreCharacter(int iChar, output_type *pOutput)
 		pOutput->szStorage = xrealloc(pOutput->szStorage,
 					pOutput->tStorageSize);
 	}
-	pOutput->szStorage[pOutput->iNextFree] = (char)iChar;
-	pOutput->szStorage[pOutput->iNextFree+1] = '\0';
+	pOutput->szStorage[pOutput->iNextFree] = (char)ucChar;
+	pOutput->szStorage[pOutput->iNextFree + 1] = '\0';
+	pOutput->iNextFree++;
+} /* end of vStoreByte */
+
+/*
+ * vStoreCharacter - store one character
+ */
+static void
+vStoreCharacter(unsigned long ulChar, output_type *pOutput)
+{
+	int	iLen;
+
+	fail(pOutput == NULL);
+
+	if (tOptions.eEncoding == encoding_utf8) {
+		fail(ulChar > 0xffff);
+		if (ulChar < 0x80) {
+			vStoreByte((unsigned char)ulChar, pOutput);
+			iLen = 1;
+		} else if (ulChar < 0x800) {
+			vStoreByte((unsigned char)(0xc0 | ulChar >> 6),
+								pOutput);
+			vStoreByte((unsigned char)(0x80 | (ulChar & 0x3f)),
+								pOutput);
+			iLen = 2;
+		} else {
+			vStoreByte((unsigned char)(0xe0 | ulChar >> 12),
+								pOutput);
+			vStoreByte((unsigned char)(0x80 | (ulChar >> 6 & 0x3f)),
+								pOutput);
+			vStoreByte((unsigned char)(0x80 | (ulChar & 0x3f)),
+								pOutput);
+			iLen = 3;
+		}
+	} else {
+		fail(ulChar > 0xff);
+		vStoreByte((unsigned char)ulChar, pOutput);
+		iLen = 1;
+	}
 	pOutput->lStringWidth += lComputeStringWidth(
-				pOutput->szStorage + pOutput->iNextFree,
-				1,
+				pOutput->szStorage + pOutput->iNextFree - iLen,
+				iLen,
 				pOutput->tFontRef,
 				pOutput->sFontsize);
-	pOutput->iNextFree++;
 } /* end of vStoreCharacter */
 
 /*
@@ -249,11 +289,12 @@ vPutIndentation(diagram_type *pDiag, output_type *pOutput, BOOL bUnmarked,
 		vSetLeftIndentation(pDiag, lLeftIndentation);
 		return;
 	}
-	fail(iListNumber <= 0 || iscntrl(cListCharacter));
+	fail(iListNumber <= 0);
+	fail(iscntrl((int)cListCharacter));
 
 	switch (ucListType) {
-	case LIST_BULLETS:
-		iNextFree = 0;
+	case LIST_ARABIC_NUM:
+		iNextFree = sprintf(szLine, "%d", iListNumber);
 		break;
 	case LIST_ROMAN_NUM_UPPER:
 	case LIST_ROMAN_NUM_LOWER:
@@ -265,9 +306,25 @@ vPutIndentation(diagram_type *pDiag, output_type *pOutput, BOOL bUnmarked,
 		iNextFree = iInteger2Alpha(iListNumber,
 			ucListType == LIST_UPPER_ALPHA, szLine);
 		break;
-	case LIST_ARABIC_NUM:
+	case LIST_ORDINAL_NUM:
+		if (iListNumber % 10 == 1 && iListNumber != 11) {
+			iNextFree = sprintf(szLine, "%dst", iListNumber);
+		} else if (iListNumber % 10 == 2 && iListNumber != 12) {
+			iNextFree = sprintf(szLine, "%dnd", iListNumber);
+		} else if (iListNumber % 10 == 3 && iListNumber != 13) {
+			iNextFree = sprintf(szLine, "%drd", iListNumber);
+		} else {
+			iNextFree = sprintf(szLine, "%dth", iListNumber);
+		}
+		break;
+	case LIST_BULLETS:
+		iNextFree = 0;
+		break;
 	default:
+		DBG_DEC(ucListType);
+		DBG_FIXME();
 		iNextFree = sprintf(szLine, "%d", iListNumber);
+		break;
 	}
 	szLine[iNextFree++] = cListCharacter;
 	szLine[iNextFree++] = ' ';
@@ -285,7 +342,7 @@ vPutIndentation(diagram_type *pDiag, output_type *pOutput, BOOL bUnmarked,
  * vPutNoteSeparator - output a note separator
  *
  * A note separator is a horizontal line two inches long.
- * Two inches equals 144000 milliponts.
+ * Two inches equals 144000 millipoints.
  */
 static void
 vPutNoteSeparator(output_type *pOutput)
@@ -332,7 +389,7 @@ pStartNextOutput(output_type *pCurrent)
 	pNew->lStringWidth = 0;
 	pNew->iColor = FONT_COLOR_DEFAULT;
 	pNew->ucFontstyle = FONT_REGULAR;
-	pNew->tFontRef = 0;
+	pNew->tFontRef = (draw_fontref)0;
 	pNew->sFontsize = DEFAULT_FONT_SIZE;
 	pNew->pPrev = pCurrent;
 	pNew->pNext = NULL;
@@ -353,7 +410,7 @@ pStartNewOutput(output_type *pAnchor, output_type *pLeftOver)
 
 	iColor = FONT_COLOR_DEFAULT;
 	ucFontstyle = FONT_REGULAR;
-	tFontRef = 0;
+	tFontRef = (draw_fontref)0;
 	sFontsize = DEFAULT_FONT_SIZE;
 	/* Free the old output space */
 	pCurr = pAnchor;
@@ -389,15 +446,18 @@ pStartNewOutput(output_type *pAnchor, output_type *pLeftOver)
 } /* end of pStartNewOutput */
 
 /*
- * iGetChar - get the next character from the given list
+ * ulGetChar - get the next character from the given list
+ *
+ * returns the next character of EOF
  */
-static int
-iGetChar(FILE *pFile, list_id_enum eListID)
+static unsigned long
+ulGetChar(FILE *pFile, list_id_enum eListID)
 {
-	const font_block_type *pCurr;
-	long	lFileOffset;
-	int	iChar;
-	unsigned short usChar;
+	const font_block_type	*pCurr;
+	unsigned long		ulChar;
+	long		lFileOffset, lTextOffset;
+	row_info_enum	eRowInfo;
+	unsigned short	usChar, usPropMod;
 	BOOL	bSkip;
 
 	fail(pFile == NULL);
@@ -405,23 +465,44 @@ iGetChar(FILE *pFile, list_id_enum eListID)
 	pCurr = pFontInfo;
 	bSkip = FALSE;
 	for (;;) {
-		usChar = usNextChar(pFile, eListID, &lFileOffset);
+		usChar = usNextChar(pFile, eListID,
+					&lFileOffset, &lTextOffset, &usPropMod);
 		if (usChar == (unsigned short)EOF) {
-			return EOF;
+			return (unsigned long)EOF;
 		}
 
 		vUpdateCounters();
 
+		eRowInfo = ePropMod2RowInfo(usPropMod, iWordVersion);
 		if (!bStartRow) {
-			bStartRow = bRowInfo &&
-				lFileOffset == tRowInfo.lOffsetStart;
-			NO_DBG_HEX_C(bStartRow, tRowInfo.lOffsetStart);
+#if 0
+			bStartRow = eRowInfo == found_a_cell ||
+				(pRowInfo != NULL &&
+				 lFileOffset == pRowInfo->lFileOffsetStart &&
+				 eRowInfo != found_not_a_cell);
+#else
+			bStartRow = pRowInfo != NULL &&
+				lFileOffset == pRowInfo->lFileOffsetStart;
+#endif
+			NO_DBG_HEX_C(bStartRow, pRowInfo->lFileOffsetStart);
 		}
-		if (!bEndRow) {
-			bEndRow = bRowInfo &&
-				lFileOffset == tRowInfo.lOffsetEnd;
-			NO_DBG_HEX_C(bStartRow, tRowInfo.lOffsetEnd);
+		if (!bEndRowNorm) {
+#if 0
+			bEndRow = eRowInfo == found_end_of_row ||
+				(pRowInfo != NULL &&
+				 lFileOffset == pRowInfo->lFileOffsetEnd &&
+				 eRowInfo != found_not_end_of_row);
+#else
+			bEndRowNorm = pRowInfo != NULL &&
+				lFileOffset == pRowInfo->lFileOffsetEnd;
+#endif
+			NO_DBG_HEX_C(bEndRowNorm, pRowInfo->lFileOffsetEnd);
 		}
+		if (!bEndRowFast) {
+			bEndRowFast = eRowInfo == found_end_of_row;
+			NO_DBG_HEX_C(bEndRowFast, pRowInfo->lFileOffsetEnd);
+		}
+
 		if (!bStartStyle) {
 			bStartStyle = pStyleInfo != NULL &&
 				lFileOffset == pStyleInfo->lFileOffset;
@@ -446,35 +527,37 @@ iGetChar(FILE *pFile, list_id_enum eListID)
 		if (bSkip) {
 			continue;
 		}
-		iChar = iTranslateCharacters(usChar, lFileOffset, bMacFile);
-		if (iChar == IGNORE_CHARACTER) {
+		ulChar = ulTranslateCharacters(usChar,
+					lFileOffset,
+					tOptions.eEncoding == encoding_utf8,
+					bWord6MacFile);
+		if (ulChar == IGNORE_CHARACTER) {
 			continue;
 		}
-		if (iChar == PICTURE) {
+		if (ulChar == PICTURE) {
 			lFileOffsetImage = lGetPicInfoListItem(lFileOffset);
 		} else {
 			lFileOffsetImage = -1;
 		}
-		return iChar;
+		return ulChar;
 	}
-} /* end of iGetChar */
+} /* end of ulGetChar */
 
 /*
  * vWord2Text
  */
 void
-vWord2Text(diagram_type *pDiag, const char *szFilename)
+vWord2Text(FILE *pFile, long lFilesize, diagram_type *pDiag)
 {
-	options_type	tOptions;
 	imagedata_type	tImage;
-	FILE	*pFile;
 	output_type	*pAnchor, *pOutput, *pLeftOver;
+	unsigned long	ulChar;
 	long	lWidthCurr, lWidthMax, lRightIndentation;
 	long	lDefaultTabWidth, lTmp;
 	list_id_enum 	eListID;
 	image_info_enum	eRes;
 	int	iFootnoteNumber, iEndnoteNumber, iLeftIndent;
-	int	iChar, iListNumber;
+	int	iListNumber;
 	BOOL	bWasTableRow, bTableFontClosed, bUnmarked;
 	BOOL	bAllCapitals, bHiddenText, bSuccess;
 	short	sFontsize;
@@ -483,31 +566,33 @@ vWord2Text(diagram_type *pDiag, const char *szFilename)
 	unsigned char	ucListType, ucAlignment;
 	char	cListChar;
 
-	fail(pDiag == NULL || szFilename == NULL || szFilename[0] == '\0');
+	fail(pFile == NULL || lFilesize <= 0 || pDiag == NULL);
 
 	DBG_MSG("vWord2Text");
 
-	pFile = pOpenDocument(szFilename);
-	if (pFile == NULL) {
+	iWordVersion = iInitDocument(pFile, lFilesize);
+	if (iWordVersion < 6) {
 		return;
 	}
 	vAddFonts2Diagram(pDiag);
 
 	/* Initialisation */
 #if defined(__riscos)
-	tCharCounter = 0;
+	lCharCounter = 0;
 	iCurrPct = 0;
 	iPrevPct = -1;
-	tDocumentLength = tGetDocumentLength();
+	lDocumentLength = lGetDocumentLength();
 #endif /* __riscos */
-	bMacFile = bFileFromTheMac();
+	bWord6MacFile = bIsWord6MacFile();
 	lDefaultTabWidth = lGetDefaultTabWidth();
 	DBG_DEC_C(lDefaultTabWidth != 36000, lDefaultTabWidth);
-	bRowInfo = bGetNextRowInfoListItem(&tRowInfo);
-	DBG_HEX_C(bRowInfo, tRowInfo.lOffsetStart);
-	DBG_HEX_C(bRowInfo, tRowInfo.lOffsetEnd);
+	pRowInfo = pGetNextRowInfoListItem();
+	DBG_HEX_C(pRowInfo != NULL, pRowInfo->lFileOffsetStart);
+	DBG_HEX_C(pRowInfo != NULL, pRowInfo->lFileOffsetEnd);
+	DBG_MSG_C(pRowInfo == NULL, "No rows at all");
 	bStartRow = FALSE;
-	bEndRow = FALSE;
+	bEndRowNorm = FALSE;
+	bEndRowFast = FALSE;
 	bIsTableRow = FALSE;
 	bWasTableRow = FALSE;
 	vResetStyles();
@@ -558,8 +643,8 @@ vWord2Text(diagram_type *pDiag, const char *szFilename)
 	iEndnoteNumber = 0;
 	eListID = text_list;
 	for(;;) {
-		iChar = iGetChar(pFile, eListID);
-		if (iChar == EOF) {
+		ulChar = ulGetChar(pFile, eListID);
+		if (ulChar == (unsigned long)EOF) {
 			if (bOutputContainsText(pAnchor)) {
 				OUTPUT_LINE();
 			} else {
@@ -593,13 +678,13 @@ vWord2Text(diagram_type *pDiag, const char *szFilename)
 			continue;
 		}
 
-		if (iChar == UNKNOWN_NOTE_CHAR) {
+		if (ulChar == UNKNOWN_NOTE_CHAR) {
 			switch (eListID) {
 			case footnote_list:
-				iChar = FOOTNOTE_CHAR;
+				ulChar = FOOTNOTE_CHAR;
 				break;
 			case endnote_list:
-				iChar = ENDNOTE_CHAR;
+				ulChar = ENDNOTE_CHAR;
 				break;
 			default:
 				break;
@@ -638,10 +723,10 @@ vWord2Text(diagram_type *pDiag, const char *szFilename)
 
 		if (bWasTableRow &&
 		    !bIsTableRow &&
-		    iChar != PAR_END &&
-		    iChar != HARD_RETURN &&
-		    iChar != FORM_FEED &&
-		    iChar != COLUMN_FEED) {
+		    ulChar != PAR_END &&
+		    ulChar != HARD_RETURN &&
+		    ulChar != FORM_FEED &&
+		    ulChar != COLUMN_FEED) {
 			/*
 			 * The end of a table should be followed by an
 			 * empty line, like the end of a paragraph
@@ -652,7 +737,7 @@ vWord2Text(diagram_type *pDiag, const char *szFilename)
 						pOutput->sFontsize);
 		}
 
-		switch (iChar) {
+		switch (ulChar) {
 		case FORM_FEED:
 		case COLUMN_FEED:
 			if (bIsTableRow) {
@@ -664,7 +749,7 @@ vWord2Text(diagram_type *pDiag, const char *szFilename)
 			} else {
 				RESET_LINE();
 			}
-			if (iChar == FORM_FEED) {
+			if (ulChar == FORM_FEED) {
 				vEndOfPage2Diagram(pDiag,
 						pOutput->tFontRef,
 						pOutput->sFontsize);
@@ -748,7 +833,7 @@ vWord2Text(diagram_type *pDiag, const char *szFilename)
 			bUnmarked = TRUE;
 		}
 
-		switch (iChar) {
+		switch (ulChar) {
 		case PICTURE:
 			(void)memset(&tImage, 0, sizeof(tImage));
 			eRes = eExamineImage(pFile, lFileOffsetImage, &tImage);
@@ -833,7 +918,7 @@ vWord2Text(diagram_type *pDiag, const char *szFilename)
 			break;
 		case TABLE_SEPARATOR:
 			if (bIsTableRow) {
-				vStoreCharacter(iChar, pOutput);
+				vStoreCharacter(ulChar, pOutput);
 				break;
 			}
 			vStoreCharacter(' ', pOutput);
@@ -865,10 +950,10 @@ vWord2Text(diagram_type *pDiag, const char *szFilename)
 			if (bHiddenText && tOptions.bHideHiddenText) {
 				continue;
 			}
-			if (bAllCapitals && iChar < UCHAR_MAX) {
-				iChar = iToUpper(iChar);
+			if (bAllCapitals) {
+				ulChar = ulToUpper(ulChar);
 			}
-			vStoreCharacter(iChar, pOutput);
+			vStoreCharacter(ulChar, pOutput);
 			break;
 		}
 
@@ -887,20 +972,27 @@ vWord2Text(diagram_type *pDiag, const char *szFilename)
 
 		if (bIsTableRow) {
 			fail(pAnchor != pOutput);
-			if (!bEndRow) {
+			if (!bEndRowNorm && !bEndRowFast) {
 				continue;
 			}
 			/* End of a table row */
-			fail(!bRowInfo);
-			vTableRow2Window(pDiag, pAnchor, &tRowInfo);
+			if (bEndRowNorm) {
+				fail(pRowInfo == NULL);
+				vTableRow2Window(pDiag, pAnchor, pRowInfo);
+			} else {
+				fail(!bEndRowFast);
+			}
 			/* Reset */
 			pAnchor = pStartNewOutput(pAnchor, NULL);
 			pOutput = pAnchor;
-			bRowInfo = bGetNextRowInfoListItem(&tRowInfo);
+			if (bEndRowNorm) {
+				pRowInfo = pGetNextRowInfoListItem();
+			}
 			bIsTableRow = FALSE;
-			bEndRow = FALSE;
-			NO_DBG_HEX_C(bRowInfo, tRowInfo.lOffsetStart);
-			NO_DBG_HEX_C(bRowInfo, tRowInfo.lOffsetEnd);
+			bEndRowNorm = FALSE;
+			bEndRowFast = FALSE;
+			DBG_HEX_C(pRowInfo != NULL, pRowInfo->lFileOffsetStart);
+			DBG_HEX_C(pRowInfo != NULL, pRowInfo->lFileOffsetEnd);
 			continue;
 		}
 		lWidthCurr = lTotalStringWidth(pAnchor);
@@ -927,6 +1019,6 @@ vWord2Text(diagram_type *pDiag, const char *szFilename)
 	pAnchor->szStorage = xfree(pAnchor->szStorage);
 	pAnchor = xfree(pAnchor);
 	vCloseFont();
-	vCloseDocument(pFile);
+	vFreeDocument();
 	visdelay_end();
 } /* end of vWord2Text */
