@@ -1,6 +1,6 @@
 /*
  * wordlib.c
- * Copyright (C) 1998-2000 A.J. van Os; Released under GPL
+ * Copyright (C) 1998-2001 A.J. van Os; Released under GPL
  *
  * Description:
  * Deal with the internals of a MS Word file
@@ -11,28 +11,28 @@
 #include <string.h>
 #include "antiword.h"
 
+/* Private type for Property Set Storage entries */
 typedef struct pps_entry_tag {
 	char	szName[32];
 	int	iType;
 	int	iNext;
 	int	iPrev;
 	int	iDir;
-	int	iSb;
+	long	lSb;
 	long	lSize;
 	int	iLevel;
 } pps_entry_type;
 
-static BOOL	bMacFile = FALSE;
 
 /* Macro to make sure all such statements will be identical */
 #define FREE_ALL()		\
 	do {\
 		vDestroySmallBlockList();\
-		aiRootList = xfree(aiRootList);\
-		aiSbdList = xfree(aiSbdList);\
-		aiBbdList = xfree(aiBbdList);\
-		aiSBD = xfree(aiSBD);\
-		aiBBD = xfree(aiBBD);\
+		alRootList = xfree(alRootList);\
+		alSbdList = xfree(alSbdList);\
+		alBbdList = xfree(alBbdList);\
+		alSBD = xfree(alSBD);\
+		alBBD = xfree(alBBD);\
 	} while(0)
 
 
@@ -81,13 +81,13 @@ vName2String(char *szName, const unsigned char *aucBytes, int iNameSize)
  * Returns the number of indices read
  */
 static size_t
-tReadBlockIndices(FILE *pFile, int *aiBlockDepot, size_t tMaxRec, long lOffset)
+tReadBlockIndices(FILE *pFile, long *alBlockDepot, size_t tMaxRec, long lOffset)
 {
 	size_t	tDone;
 	int	iIndex;
 	unsigned char	aucBytes[BIG_BLOCK_SIZE];
 
-	fail(pFile == NULL || aiBlockDepot == NULL);
+	fail(pFile == NULL || alBlockDepot == NULL);
 	fail(tMaxRec == 0);
 	fail(lOffset < 0);
 
@@ -97,10 +97,10 @@ tReadBlockIndices(FILE *pFile, int *aiBlockDepot, size_t tMaxRec, long lOffset)
 		return 0;
 	}
 	/* Split the big block into indices, an index is four bytes */
-	tDone = min(tMaxRec, BIG_BLOCK_SIZE / 4);
+	tDone = min(tMaxRec, (size_t)BIG_BLOCK_SIZE / 4);
 	for (iIndex = 0; iIndex < (int)tDone; iIndex++) {
-		aiBlockDepot[iIndex] = (int)ulGetLong(4 * iIndex, aucBytes);
-		NO_DBG_DEC(aiBlockDepot[iIndex]);
+		alBlockDepot[iIndex] = (long)ulGetLong(4 * iIndex, aucBytes);
+		NO_DBG_DEC(alBlockDepot[iIndex]);
 	}
 	return tDone;
 } /* end of tReadBlockIndices */
@@ -109,27 +109,27 @@ tReadBlockIndices(FILE *pFile, int *aiBlockDepot, size_t tMaxRec, long lOffset)
  * bGetBBD - get the Big Block Depot indices from the index-blocks
  */
 static BOOL
-bGetBBD(FILE *pFile, const int *aiDepot, size_t tDepotLen,
-			int *aiBBD, size_t tBBDLen)
+bGetBBD(FILE *pFile, const long *alDepot, size_t tDepotLen,
+	long *alBBD, size_t tBBDLen)
 {
 	long	lBegin;
 	size_t	tToGo, tDone;
 	int	iIndex;
 
-	fail(pFile == NULL || aiDepot == NULL || aiBBD == NULL);
+	fail(pFile == NULL || alDepot == NULL || alBBD == NULL);
 
 	DBG_MSG("bGetBBD");
 
 	tToGo = tBBDLen;
 	for (iIndex = 0; iIndex < (int)tDepotLen && tToGo != 0; iIndex++) {
-		lBegin = ((long)aiDepot[iIndex] + 1) * BIG_BLOCK_SIZE;
+		lBegin = (alDepot[iIndex] + 1) * BIG_BLOCK_SIZE;
 		NO_DBG_HEX(lBegin);
-		tDone = tReadBlockIndices(pFile, aiBBD, tToGo, lBegin);
+		tDone = tReadBlockIndices(pFile, alBBD, tToGo, lBegin);
 		fail(tDone > tToGo);
 		if (tDone == 0) {
 			return FALSE;
 		}
-		aiBBD += tDone;
+		alBBD += tDone;
 		tToGo -= tDone;
 	}
 	return tToGo == 0;
@@ -139,27 +139,27 @@ bGetBBD(FILE *pFile, const int *aiDepot, size_t tDepotLen,
  * bGetSBD - get the Small Block Depot indices from the index-blocks
  */
 static BOOL
-bGetSBD(FILE *pFile, const int *aiDepot, size_t tDepotLen,
-			int *aiSBD, size_t tSBDLen)
+bGetSBD(FILE *pFile, const long *alDepot, size_t tDepotLen,
+	long *alSBD, size_t tSBDLen)
 {
 	long	lBegin;
 	size_t	tToGo, tDone;
 	int	iIndex;
 
-	fail(pFile == NULL || aiDepot == NULL || aiSBD == NULL);
+	fail(pFile == NULL || alDepot == NULL || alSBD == NULL);
 
 	DBG_MSG("bGetSBD");
 
 	tToGo = tSBDLen;
 	for (iIndex = 0; iIndex < (int)tDepotLen && tToGo != 0; iIndex++) {
-		lBegin = ((long)aiDepot[iIndex] + 1) * BIG_BLOCK_SIZE;
+		lBegin = (alDepot[iIndex] + 1) * BIG_BLOCK_SIZE;
 		NO_DBG_HEX(lBegin);
-		tDone = tReadBlockIndices(pFile, aiSBD, tToGo, lBegin);
+		tDone = tReadBlockIndices(pFile, alSBD, tToGo, lBegin);
 		fail(tDone > tToGo);
 		if (tDone == 0) {
 			return FALSE;
 		}
-		aiSBD += tDone;
+		alSBD += tDone;
 		tToGo -= tDone;
 	}
 	return tToGo == 0;
@@ -216,18 +216,17 @@ vComputePPSlevels(pps_entry_type *atPPSlist, pps_entry_type *pNode,
  */
 static BOOL
 bGetPPS(FILE *pFile,
-		const int *aiRootList, size_t tRootListLen,
-		pps_info_type *pPPS)
+	const long *alRootList, size_t tRootListLen, pps_info_type *pPPS)
 {
 	pps_entry_type	*atPPSlist;
-	long	lBegin;
+	long	lBegin, lTmp;
 	size_t	tNbrOfPPS;
-	int	iTmp, iIndex, iStartBlock, iOffset;
+	int	iIndex, iStartBlock, iOffset;
 	int	iNameSize, iRootIndex;
 	BOOL	bWord, bExcel;
 	unsigned char	aucBytes[PROPERTY_SET_STORAGE_SIZE];
 
-	fail(pFile == NULL || pPPS == NULL || aiRootList == NULL);
+	fail(pFile == NULL || pPPS == NULL || alRootList == NULL);
 
 	DBG_MSG("bGetPPS");
 	NO_DBG_DEC(tRootListLen);
@@ -241,10 +240,10 @@ bGetPPS(FILE *pFile,
 	atPPSlist = xmalloc(tNbrOfPPS * sizeof(pps_entry_type));
 	iRootIndex = 0;
 	for (iIndex = 0; iIndex < (int)tNbrOfPPS; iIndex++) {
-		iTmp = iIndex * PROPERTY_SET_STORAGE_SIZE;
-		iStartBlock = iTmp / BIG_BLOCK_SIZE;
-		iOffset = iTmp % BIG_BLOCK_SIZE;
-		lBegin = ((long)aiRootList[iStartBlock] + 1) * BIG_BLOCK_SIZE +
+		lTmp = (long)iIndex * PROPERTY_SET_STORAGE_SIZE;
+		iStartBlock = (int)(lTmp / BIG_BLOCK_SIZE);
+		iOffset = (int)(lTmp % BIG_BLOCK_SIZE);
+		lBegin = (alRootList[iStartBlock] + 1) * BIG_BLOCK_SIZE +
 			iOffset;
 		NO_DBG_HEX(lBegin);
 		if (!bReadBytes(aucBytes, PROPERTY_SET_STORAGE_SIZE,
@@ -263,7 +262,7 @@ bGetPPS(FILE *pFile,
 		atPPSlist[iIndex].iPrev = (int)ulGetLong(0x44, aucBytes);
 		atPPSlist[iIndex].iNext = (int)ulGetLong(0x48, aucBytes);
 		atPPSlist[iIndex].iDir = (int)ulGetLong(0x4c, aucBytes);
-		atPPSlist[iIndex].iSb = (int)ulGetLong(0x74, aucBytes);
+		atPPSlist[iIndex].lSb = (long)ulGetLong(0x74, aucBytes);
 		atPPSlist[iIndex].lSize = (long)ulGetLong(0x78, aucBytes);
 		atPPSlist[iIndex].iLevel = INT_MAX;
 		if (atPPSlist[iIndex].iPrev < -1 ||
@@ -319,20 +318,20 @@ bGetPPS(FILE *pFile,
 		}
 		if (pPPS->tWordDocument.lSize <= 0 &&
 		    STREQ(atPPSlist[iIndex].szName, "WordDocument")) {
-			pPPS->tWordDocument.iSb = atPPSlist[iIndex].iSb;
+			pPPS->tWordDocument.lSb = atPPSlist[iIndex].lSb;
 			pPPS->tWordDocument.lSize = atPPSlist[iIndex].lSize;
 			bWord = TRUE;
 		} else if (pPPS->tData.lSize <= 0 &&
 			   STREQ(atPPSlist[iIndex].szName, "Data")) {
-			pPPS->tData.iSb = atPPSlist[iIndex].iSb;
+			pPPS->tData.lSb = atPPSlist[iIndex].lSb;
 			pPPS->tData.lSize = atPPSlist[iIndex].lSize;
 		} else if (pPPS->t0Table.lSize <= 0 &&
 			   STREQ(atPPSlist[iIndex].szName, "0Table")) {
-			pPPS->t0Table.iSb = atPPSlist[iIndex].iSb;
+			pPPS->t0Table.lSb = atPPSlist[iIndex].lSb;
 			pPPS->t0Table.lSize = atPPSlist[iIndex].lSize;
 		} else if (pPPS->t1Table.lSize <= 0 &&
 			   STREQ(atPPSlist[iIndex].szName, "1Table")) {
-			pPPS->t1Table.iSb = atPPSlist[iIndex].iSb;
+			pPPS->t1Table.lSb = atPPSlist[iIndex].lSb;
 			pPPS->t1Table.lSize = atPPSlist[iIndex].lSize;
 		} else if (STREQ(atPPSlist[iIndex].szName, "Book") ||
 			   STREQ(atPPSlist[iIndex].szName, "Workbook")) {
@@ -355,6 +354,28 @@ bGetPPS(FILE *pFile,
 	return FALSE;
 } /* end of bGetPPS */
 
+/*
+ * vGetBbdList - make a list of the places to find big blocks
+ */
+static void
+vGetBbdList(FILE *pFile, int iNbr, long *alBbdList, long lOffset)
+{
+	int	iIndex;
+
+	fail(pFile == NULL);
+	fail(iNbr > 127);
+	fail(alBbdList == NULL);
+	fail(lOffset < 0);
+
+	NO_DBG_DEC(iNbr);
+	for (iIndex = 0; iIndex < iNbr; iIndex++) {
+                alBbdList[iIndex] =
+                        (long)ulReadLong(pFile, lOffset + 4 * (long)iIndex);
+		NO_DBG_DEC(iIndex);
+                NO_DBG_HEX(alBbdList[iIndex]);
+        }
+} /* end of vGetBbdList */
+
 /*
  * bGetDocumentText - make a list of the text blocks of a Word document
  *
@@ -362,21 +383,21 @@ bGetPPS(FILE *pFile,
  */
 static BOOL
 bGetDocumentText(FILE *pFile, const pps_info_type *pPPS,
-		const int *aiBBD, size_t tBBDLen,
-		const int *aiSBD, size_t tSBDLen,
-		const unsigned char *aucHeader, int iWordVersion)
+	const long *alBBD, size_t tBBDLen,
+	const long *alSBD, size_t tSBDLen,
+	const unsigned char *aucHeader, int iWordVersion)
 {
 	long	lBeginOfText;
-	size_t	tTextLen, tFootnoteLen, tEndnoteLen;
-	size_t	tHeaderLen, tMacroLen, tAnnotationLen;
-	size_t	tTextBoxLen, tHdrTextBoxLen;
+	long	lTextLen, lFootnoteLen, lEndnoteLen;
+	long	lHeaderLen, lMacroLen, lAnnotationLen;
+	long	lTextBoxLen, lHdrTextBoxLen;
+	unsigned int	uiQuickSaves;
 	BOOL	bFarEastWord, bFastSaved, bEncrypted, bSuccess;
-	text_info_enum	eResult;
 	unsigned short	usDocStatus, usIdent;
 
 	fail(pFile == NULL || pPPS == NULL);
-	fail(aiBBD == NULL);
-	fail(aiSBD == NULL);
+	fail(alBBD == NULL);
+	fail(alSBD == NULL);
 
 	DBG_MSG("bGetDocumentText");
 
@@ -389,8 +410,9 @@ bGetDocumentText(FILE *pFile, const pps_info_type *pPPS,
 	usDocStatus = usGetWord(0x0a, aucHeader);
 	DBG_HEX(usDocStatus);
 	bFastSaved = (usDocStatus & BIT(2)) != 0;
+	uiQuickSaves = (usDocStatus & 0x00f0) >> 4;
 	DBG_MSG_C(bFastSaved, "This document is Fast Saved");
-	DBG_DEC_C(bFastSaved, (usDocStatus & 0x00f0) >> 4);
+	DBG_DEC_C(bFastSaved, uiQuickSaves);
 	bEncrypted = (usDocStatus & BIT(8)) != 0;
 	if (bEncrypted) {
 		werr(0, "Encrypted documents are not supported");
@@ -401,75 +423,61 @@ bGetDocumentText(FILE *pFile, const pps_info_type *pPPS,
 	lBeginOfText = (long)ulGetLong(0x18, aucHeader);
 	DBG_HEX(lBeginOfText);
 	if (iWordVersion == 6 || iWordVersion == 7) {
-		tTextLen = (size_t)ulGetLong(0x34, aucHeader);
-		tFootnoteLen = (size_t)ulGetLong(0x38, aucHeader);
-		tHeaderLen = (size_t)ulGetLong(0x3c, aucHeader);
-		tMacroLen = (size_t)ulGetLong(0x40, aucHeader);
-		tAnnotationLen = (size_t)ulGetLong(0x44, aucHeader);
-		tEndnoteLen = (size_t)ulGetLong(0x48, aucHeader);
-		tTextBoxLen = (size_t)ulGetLong(0x4c, aucHeader);
-		tHdrTextBoxLen = (size_t)ulGetLong(0x50, aucHeader);
+		lTextLen = (long)ulGetLong(0x34, aucHeader);
+		lFootnoteLen = (long)ulGetLong(0x38, aucHeader);
+		lHeaderLen = (long)ulGetLong(0x3c, aucHeader);
+		lMacroLen = (long)ulGetLong(0x40, aucHeader);
+		lAnnotationLen = (long)ulGetLong(0x44, aucHeader);
+		lEndnoteLen = (long)ulGetLong(0x48, aucHeader);
+		lTextBoxLen = (long)ulGetLong(0x4c, aucHeader);
+		lHdrTextBoxLen = (long)ulGetLong(0x50, aucHeader);
 	} else {
-		tTextLen = (size_t)ulGetLong(0x4c, aucHeader);
-		tFootnoteLen = (size_t)ulGetLong(0x50, aucHeader);
-		tHeaderLen = (size_t)ulGetLong(0x54, aucHeader);
-		tMacroLen = (size_t)ulGetLong(0x58, aucHeader);
-		tAnnotationLen = (size_t)ulGetLong(0x5c, aucHeader);
-		tEndnoteLen = (size_t)ulGetLong(0x60, aucHeader);
-		tTextBoxLen = (size_t)ulGetLong(0x64, aucHeader);
-		tHdrTextBoxLen = (size_t)ulGetLong(0x68, aucHeader);
+		lTextLen = (long)ulGetLong(0x4c, aucHeader);
+		lFootnoteLen = (long)ulGetLong(0x50, aucHeader);
+		lHeaderLen = (long)ulGetLong(0x54, aucHeader);
+		lMacroLen = (long)ulGetLong(0x58, aucHeader);
+		lAnnotationLen = (long)ulGetLong(0x5c, aucHeader);
+		lEndnoteLen = (long)ulGetLong(0x60, aucHeader);
+		lTextBoxLen = (long)ulGetLong(0x64, aucHeader);
+		lHdrTextBoxLen = (long)ulGetLong(0x68, aucHeader);
 	}
-	DBG_DEC(tTextLen);
-	DBG_DEC(tFootnoteLen);
-	DBG_DEC(tHeaderLen);
-	DBG_DEC(tMacroLen);
-	DBG_DEC(tAnnotationLen);
-	DBG_DEC(tEndnoteLen);
-	DBG_DEC(tTextBoxLen);
-	DBG_DEC(tHdrTextBoxLen);
+	DBG_DEC(lTextLen);
+	DBG_DEC(lFootnoteLen);
+	DBG_DEC(lHeaderLen);
+	DBG_DEC(lMacroLen);
+	DBG_DEC(lAnnotationLen);
+	DBG_DEC(lEndnoteLen);
+	DBG_DEC(lTextBoxLen);
+	DBG_DEC(lHdrTextBoxLen);
 
 	/* Make a list of the text blocks */
 	switch (iWordVersion) {
 	case 6:
 	case 7:
 		if (bFastSaved) {
-			eResult = eGet6DocumentText(pFile,
+			bSuccess = bGet6DocumentText(pFile,
 					bFarEastWord,
-					pPPS->tWordDocument.iSb,
-					aiBBD, tBBDLen,
+					pPPS->tWordDocument.lSb,
+					alBBD, tBBDLen,
 					aucHeader);
-			bSuccess = eResult == text_success;
 		} else {
 		  	bSuccess = bAddTextBlocks(lBeginOfText,
-				tTextLen +
-				tFootnoteLen +
-				tHeaderLen + tMacroLen + tAnnotationLen +
-				tEndnoteLen +
-				tTextBoxLen + tHdrTextBoxLen,
+				lTextLen +
+				lFootnoteLen +
+				lHeaderLen + lMacroLen + lAnnotationLen +
+				lEndnoteLen +
+				lTextBoxLen + lHdrTextBoxLen,
 				bFarEastWord,
-				pPPS->tWordDocument.iSb,
-				aiBBD, tBBDLen);
+				IGNORE_PROPMOD,
+				pPPS->tWordDocument.lSb,
+				alBBD, tBBDLen);
 		}
 		break;
 	case 8:
-		eResult = eGet8DocumentText(pFile,
+		bSuccess = bGet8DocumentText(pFile,
 				pPPS,
-				aiBBD, tBBDLen, aiSBD, tSBDLen,
+				alBBD, tBBDLen, alSBD, tSBDLen,
 				aucHeader);
-		if (eResult == text_no_information && !bFastSaved) {
-			/* Assume there is only one block */
-			bSuccess = bAddTextBlocks(lBeginOfText,
-				tTextLen +
-				tFootnoteLen +
-				tHeaderLen + tMacroLen + tAnnotationLen +
-				tEndnoteLen +
-				tTextBoxLen + tHdrTextBoxLen,
-				FALSE,
-				pPPS->tWordDocument.iSb,
-				aiBBD, tBBDLen);
-		} else {
-			bSuccess = eResult == text_success;
-		}
 		break;
 	default:
 		werr(0, "This version of Word is not supported");
@@ -478,11 +486,11 @@ bGetDocumentText(FILE *pFile, const pps_info_type *pPPS,
 	}
 
 	if (bSuccess) {
-		vSplitBlockList(tTextLen,
-				tFootnoteLen,
-				tHeaderLen + tMacroLen + tAnnotationLen,
-				tEndnoteLen,
-				tTextBoxLen + tHdrTextBoxLen,
+		vSplitBlockList(lTextLen,
+				lFootnoteLen,
+				lHeaderLen + lMacroLen + lAnnotationLen,
+				lEndnoteLen,
+				lTextBoxLen + lHdrTextBoxLen,
 				!bFastSaved && iWordVersion == 8);
 	} else {
 		vDestroyTextBlockList();
@@ -496,8 +504,8 @@ bGetDocumentText(FILE *pFile, const pps_info_type *pPPS,
  */
 static void
 vGetDocumentData(FILE *pFile, const pps_info_type *pPPS,
-		const int *aiBBD, size_t tBBDLen,
-		const unsigned char *aucHeader, int iWordVersion)
+	const long *alBBD, size_t tBBDLen,
+	const unsigned char *aucHeader, int iWordVersion)
 {
 	options_type	tOptions;
 	long	lBeginOfText;
@@ -506,7 +514,7 @@ vGetDocumentData(FILE *pFile, const pps_info_type *pPPS,
 
 	fail(pFile == NULL);
 	fail(pPPS == NULL);
-	fail(aiBBD == NULL);
+	fail(alBBD == NULL);
 
 	/* Get the options */
 	vGetOptions(&tOptions);
@@ -542,12 +550,12 @@ vGetDocumentData(FILE *pFile, const pps_info_type *pPPS,
 		 */
 		if (bFastSaved) {
 			bSuccess = bGet6DocumentData(pFile,
-					pPPS->tWordDocument.iSb,
-					aiBBD, tBBDLen,
+					pPPS->tWordDocument.lSb,
+					alBBD, tBBDLen,
 					aucHeader);
 		} else {
-		  	bSuccess = bAddDataBlocks(lBeginOfText, SIZE_T_MAX,
-				pPPS->tWordDocument.iSb, aiBBD, tBBDLen);
+		  	bSuccess = bAddDataBlocks(lBeginOfText, LONG_MAX,
+				pPPS->tWordDocument.lSb, alBBD, tBBDLen);
 		}
 		break;
 	case 8:
@@ -555,8 +563,8 @@ vGetDocumentData(FILE *pFile, const pps_info_type *pPPS,
 		 * The data blocks are in the data stream. The data stream
 		 * is always in "normal saved" format
 		 */
-		bSuccess = bAddDataBlocks(0, SIZE_T_MAX,
-				pPPS->tData.iSb, aiBBD, tBBDLen);
+		bSuccess = bAddDataBlocks(0, LONG_MAX,
+				pPPS->tData.lSb, alBBD, tBBDLen);
 		break;
 	default:
 		werr(0, "This version of Word is not supported");
@@ -571,37 +579,24 @@ vGetDocumentData(FILE *pFile, const pps_info_type *pPPS,
 } /* end of vGetDocumentData */
 
 /*
- * pInitFile - open and check the file
+ * iInitDocument - initialize the document
+ *
+ * Returns the version of Word that made the document or -1
  */
-static FILE *
-pInitFile(const char *szFilename)
-{
-	FILE	*pFile;
-
-	fail(szFilename == NULL || szFilename[0] == '\0');
-
-	pFile = fopen(szFilename, "rb");
-	if (pFile == NULL) {
-		werr(0, "I can't open '%s' for reading", szFilename);
-		return NULL;
-	}
-	return pFile;
-} /* end of pInitFile */
-
-static BOOL
-bInitDoc(FILE *pFile, long lFilesize)
+int
+iInitDocument(FILE *pFile, long lFilesize)
 {
 	pps_info_type	PPS_info;
-	int	*aiBBD, *aiRootList, *aiBbdList;
-	int	*aiSBD, *aiSbdList;
-	long	lMaxBlock;
-	size_t	tBBDLen, tSBDLen, tNumBbdBlocks, tRootListLen, tSize;
-	int	iRootStartblock;
-	int	iSbdStartblock, iSBLstartblock;
-	int	iWordVersion, iIndex, iTmp;
+	long	*alBBD, *alSBD;
+	long	*alRootList, *alBbdList, *alSbdList;
+	long	lRootStartblock, lSbdStartblock, lSBLstartblock;
+	long	lAdditionalBBDlist;
+	long	lBdbListStart, lMaxBlock, lTmp;
+	size_t	tBBDLen, tSBDLen, tNumBbdBlocks, tRootListLen;
+	int	iWordVersion, iIndex, iStart, iToGo;
 	int	iMaxSmallBlock;
-	BOOL	bResult;
-	unsigned short	usIdent, usFib, usChse;
+	BOOL	bSuccess;
+	unsigned short	usIdent;
 	unsigned char	aucHeader[HEADER_SIZE];
 
 	fail(pFile == NULL);
@@ -609,107 +604,112 @@ bInitDoc(FILE *pFile, long lFilesize)
 	lMaxBlock = lFilesize / BIG_BLOCK_SIZE - 2;
 	DBG_DEC(lMaxBlock);
 	if (lMaxBlock < 1) {
-		return FALSE;
+		return -1;
 	}
 	tBBDLen = (size_t)(lMaxBlock + 1);
 	tNumBbdBlocks = (size_t)ulReadLong(pFile, 0x2c);
 	DBG_DEC(tNumBbdBlocks);
-	if (tNumBbdBlocks > 109) {
-		DBG_DEC(lFilesize);
-		if (lFilesize > 109 * (BIG_BLOCK_SIZE/4) * BIG_BLOCK_SIZE) {
-			werr(0, "I'm afraid this file is too big to handle.");
-		} else {
-			werr(0, "This file is probably damaged. Sorry.");
-		}
-		return FALSE;
-	}
-	iRootStartblock = (int)ulReadLong(pFile, 0x30);
-	DBG_DEC(iRootStartblock);
-	iSbdStartblock = (int)ulReadLong(pFile, 0x3c);
-	DBG_DEC(iSbdStartblock);
-	iSBLstartblock = (int)ulReadLong(pFile,
-		((long)iRootStartblock + 1) * BIG_BLOCK_SIZE + 0x74);
-	DBG_DEC(iSBLstartblock);
+	lRootStartblock = (long)ulReadLong(pFile, 0x30);
+	DBG_DEC(lRootStartblock);
+	lSbdStartblock = (long)ulReadLong(pFile, 0x3c);
+	DBG_DEC(lSbdStartblock);
+	lAdditionalBBDlist = (long)ulReadLong(pFile, 0x44);
+	DBG_DEC(lAdditionalBBDlist);
+	DBG_HEX(lAdditionalBBDlist);
+	lSBLstartblock = (long)ulReadLong(pFile,
+		(lRootStartblock + 1) * BIG_BLOCK_SIZE + 0x74);
+	DBG_DEC(lSBLstartblock);
 	iMaxSmallBlock = (int)(ulReadLong(pFile,
-		((long)iRootStartblock + 1) *
+		(lRootStartblock + 1) *
 		BIG_BLOCK_SIZE + 0x78) / SMALL_BLOCK_SIZE) - 1;
 	DBG_DEC(iMaxSmallBlock);
 	tSBDLen = (size_t)(iMaxSmallBlock + 1);
 	/* All to be xmalloc-ed pointers to NULL */
-	aiRootList = NULL;
-	aiSbdList = NULL;
-	aiBbdList = NULL;
-	aiSBD = NULL;
-	aiBBD = NULL;
+	alRootList = NULL;
+	alSbdList = NULL;
+	alBbdList = NULL;
+	alSBD = NULL;
+	alBBD = NULL;
 /* Big Block Depot */
-	tSize = tNumBbdBlocks * sizeof(int);
-	aiBbdList = xmalloc(tSize);
-	tSize = tBBDLen * sizeof(int);
-	aiBBD = xmalloc(tSize);
-	for (iIndex = 0; iIndex < (int)tNumBbdBlocks; iIndex++) {
-		aiBbdList[iIndex] =
-			(int)ulReadLong(pFile, 0x4c + 4 * (long)iIndex);
-		NO_DBG_HEX(aiBbdList[iIndex]);
+	alBbdList = xmalloc(tNumBbdBlocks * sizeof(long));
+	alBBD = xmalloc(tBBDLen * sizeof(long));
+	iToGo = (int)tNumBbdBlocks;
+	vGetBbdList(pFile, min(iToGo, 109),  alBbdList, 0x4c);
+	iStart = 109;
+	iToGo -= 109;
+	while (lAdditionalBBDlist != END_OF_CHAIN && iToGo > 0) {
+		lBdbListStart = (lAdditionalBBDlist + 1) * BIG_BLOCK_SIZE;
+		vGetBbdList(pFile, min(iToGo, 127),
+					alBbdList + iStart, lBdbListStart);
+		lAdditionalBBDlist = (long)ulReadLong(pFile,
+					lBdbListStart + 4 * 127);
+		DBG_DEC(lAdditionalBBDlist);
+		DBG_HEX(lAdditionalBBDlist);
+		iStart += 127;
+		iToGo -= 127;
 	}
-	if (!bGetBBD(pFile, aiBbdList, tNumBbdBlocks, aiBBD, tBBDLen)) {
+	if (!bGetBBD(pFile, alBbdList, tNumBbdBlocks, alBBD, tBBDLen)) {
 		FREE_ALL();
-		return FALSE;
+		return -1;
 	}
-	aiBbdList = xfree(aiBbdList);
+	alBbdList = xfree(alBbdList);
 /* Small Block Depot */
-	tSize = tBBDLen * sizeof(int);
-	aiSbdList = xmalloc(tSize);
-	tSize = tSBDLen * sizeof(int);
-	aiSBD = xmalloc(tSize);
-	for (iIndex = 0, iTmp = iSbdStartblock;
-	     iIndex < (int)tBBDLen && iTmp != END_OF_CHAIN;
-	     iIndex++, iTmp = aiBBD[iTmp]) {
-		if (iTmp < 0 || iTmp >= (int)tBBDLen) {
+	alSbdList = xmalloc(tBBDLen * sizeof(long));
+	alSBD = xmalloc(tSBDLen * sizeof(long));
+	for (iIndex = 0, lTmp = lSbdStartblock;
+	     iIndex < (int)tBBDLen && lTmp != END_OF_CHAIN;
+	     iIndex++, lTmp = alBBD[lTmp]) {
+		if (lTmp < 0 || lTmp >= (long)tBBDLen) {
+			DBG_DEC(lTmp);
+			DBG_DEC(tBBDLen);
 			werr(1, "The Big Block Depot is damaged");
 		}
-		aiSbdList[iIndex] = iTmp;
-		NO_DBG_HEX(aiSbdList[iIndex]);
+		alSbdList[iIndex] = lTmp;
+		NO_DBG_HEX(alSbdList[iIndex]);
 	}
-	if (!bGetSBD(pFile, aiSbdList, tBBDLen, aiSBD, tSBDLen)) {
+	if (!bGetSBD(pFile, alSbdList, tBBDLen, alSBD, tSBDLen)) {
 		FREE_ALL();
-		return FALSE;
+		return -1;
 	}
-	aiSbdList = xfree(aiSbdList);
+	alSbdList = xfree(alSbdList);
 /* Root list */
-	for (tRootListLen = 0, iTmp = iRootStartblock;
-	     tRootListLen < tBBDLen && iTmp != END_OF_CHAIN;
-	     tRootListLen++, iTmp = aiBBD[iTmp]) {
-		if (iTmp < 0 || iTmp >= (int)tBBDLen) {
+	for (tRootListLen = 0, lTmp = lRootStartblock;
+	     tRootListLen < tBBDLen && lTmp != END_OF_CHAIN;
+	     tRootListLen++, lTmp = alBBD[lTmp]) {
+		if (lTmp < 0 || lTmp >= (long)tBBDLen) {
+			DBG_DEC(lTmp);
+			DBG_DEC(tBBDLen);
 			werr(1, "The Big Block Depot is damaged");
 		}
 	}
 	if (tRootListLen == 0) {
 		werr(0, "No Rootlist found");
 		FREE_ALL();
-		return FALSE;
+		return -1;
 	}
-	tSize = tRootListLen * sizeof(int);
-	aiRootList = xmalloc(tSize);
-	for (iIndex = 0, iTmp = iRootStartblock;
-	     iIndex < (int)tBBDLen && iTmp != END_OF_CHAIN;
-	     iIndex++, iTmp = aiBBD[iTmp]) {
-		if (iTmp < 0 || iTmp >= (int)tBBDLen) {
+	alRootList = xmalloc(tRootListLen * sizeof(long));
+	for (iIndex = 0, lTmp = lRootStartblock;
+	     iIndex < (int)tBBDLen && lTmp != END_OF_CHAIN;
+	     iIndex++, lTmp = alBBD[lTmp]) {
+		if (lTmp < 0 || lTmp >= (long)tBBDLen) {
+			DBG_DEC(lTmp);
+			DBG_DEC(tBBDLen);
 			werr(1, "The Big Block Depot is damaged");
 		}
-		aiRootList[iIndex] = iTmp;
-		NO_DBG_DEC(aiRootList[iIndex]);
+		alRootList[iIndex] = lTmp;
+		NO_DBG_DEC(alRootList[iIndex]);
 	}
 	fail(tRootListLen != (size_t)iIndex);
-	bResult = bGetPPS(pFile, aiRootList, tRootListLen, &PPS_info);
-	aiRootList = xfree(aiRootList);
-	if (!bResult) {
+	bSuccess = bGetPPS(pFile, alRootList, tRootListLen, &PPS_info);
+	alRootList = xfree(alRootList);
+	if (!bSuccess) {
 		FREE_ALL();
-		return FALSE;
+		return -1;
 	}
 /* Small block list */
-	if (!bCreateSmallBlockList(iSBLstartblock, aiBBD, tBBDLen)) {
+	if (!bCreateSmallBlockList(lSBLstartblock, alBBD, tBBDLen)) {
 		FREE_ALL();
-		return FALSE;
+		return -1;
 	}
 
 	if (PPS_info.tWordDocument.lSize < MIN_SIZE_FOR_BBD_USE) {
@@ -717,14 +717,14 @@ bInitDoc(FILE *pFile, long lFilesize)
 		FREE_ALL();
 		werr(0, "I'm afraid the text stream of this file "
 			"is too small to handle.");
-		return FALSE;
+		return -1;
 	}
 	/* Read the headerblock */
-	if (!bReadBuffer(pFile, PPS_info.tWordDocument.iSb,
-			aiBBD, tBBDLen, BIG_BLOCK_SIZE,
+	if (!bReadBuffer(pFile, PPS_info.tWordDocument.lSb,
+			alBBD, tBBDLen, BIG_BLOCK_SIZE,
 			aucHeader, 0, HEADER_SIZE)) {
 		FREE_ALL();
-		return FALSE;
+		return -1;
 	}
 	usIdent = usGetWord(0x00, aucHeader);
 	DBG_HEX(usIdent);
@@ -734,106 +734,40 @@ bInitDoc(FILE *pFile, long lFilesize)
 	     usIdent != 0xa5ec &&	/* Word 7 & 97 & 98 */
 	     usIdent != 0xa697 &&	/* Word 7 for oriental languages */
 	     usIdent != 0xa699);	/* Word 7 for oriental languages */
-	usFib = usGetWord(0x02, aucHeader);
-	DBG_DEC(usFib);
-	if (usFib < 101) {
+	iWordVersion = iGetVersionNumber(aucHeader);
+	if (iWordVersion < 6) {
 		FREE_ALL();
 		werr(0, "This file is from a version of Word before Word 6.");
-		return FALSE;
+		return -1;
 	}
-	usChse = usGetWord(0x14, aucHeader);
-	DBG_DEC(usChse);
-	bMacFile = FALSE;
-	/* Get the version of Word */
-	switch (usFib) {
-	case 101:
-	case 102:
-		iWordVersion = 6;
-		DBG_MSG("Word 6 for Windows");
-		break;
-	case 103:
-	case 104:
-		switch (usChse) {
-		case 0:
-			iWordVersion = 7;
-			DBG_MSG("Word 7 for Win95");
-			break;
-		case 256:
-			iWordVersion = 6;
-			DBG_MSG("Word 6 for Macintosh");
-			bMacFile = TRUE;
-			break;
-		default:
-			if (ucGetByte(0x05, aucHeader) == 0xe0) {
-				iWordVersion = 7;
-				DBG_MSG("Word 7 for Win95");
-			} else {
-				iWordVersion = 6;
-				DBG_MSG("Word 6 for Macintosh");
-				bMacFile = TRUE;
-			}
-		}
-		break;
-	default:
-		iWordVersion = 8;
-		DBG_MSG_C(usChse != 256, "Word97 for Win95/98/NT");
-		DBG_MSG_C(usChse == 256, "Word98 for Macintosh");
-		/* NOTE: No Macintosh character set !! */
-		break;
-	}
-	bResult = bGetDocumentText(pFile, &PPS_info,
-			aiBBD, tBBDLen, aiSBD, tSBDLen,
+
+	bSuccess = bGetDocumentText(pFile, &PPS_info,
+			alBBD, tBBDLen, alSBD, tSBDLen,
 			aucHeader, iWordVersion);
-	if (bResult) {
+	if (bSuccess) {
 		vGetDocumentData(pFile, &PPS_info,
-			aiBBD, tBBDLen, aucHeader, iWordVersion);
+			alBBD, tBBDLen, aucHeader, iWordVersion);
 		vSetDefaultTabWidth(pFile, &PPS_info,
-			aiBBD, tBBDLen, aiSBD, tSBDLen,
+			alBBD, tBBDLen, alSBD, tSBDLen,
 			aucHeader, iWordVersion);
-		vGetParagraphInfo(pFile, &PPS_info,
-			aiBBD, tBBDLen, aiSBD, tSBDLen,
+		vGetPropertyInfo(pFile, &PPS_info,
+			alBBD, tBBDLen, alSBD, tSBDLen,
 			aucHeader, iWordVersion);
 		vGetNotesInfo(pFile, &PPS_info,
-			aiBBD, tBBDLen, aiSBD, tSBDLen,
+			alBBD, tBBDLen, alSBD, tSBDLen,
 			aucHeader, iWordVersion);
 	}
 	FREE_ALL();
-	return bResult;
-} /* end of bInitDoc */
+	return bSuccess ? iWordVersion : -1;
+} /* end of iInitDocument */
 
 /*
- * pOpenDocument - open and initialize the document
- *
- * Returns the pointer to the document file, or NULL in case of failure
- */
-FILE *
-pOpenDocument(const char *szFilename)
-{
-	FILE	*pFile;
-	long	lFilesize;
-
-	lFilesize = lGetFilesize(szFilename);
-	if (lFilesize <= 0) {
-		return NULL;
-	}
-	pFile = pInitFile(szFilename);
-	if (pFile == NULL) {
-		return NULL;
-	}
-	if (bInitDoc(pFile, lFilesize)) {
-		return pFile;
-	}
-	(void)fclose(pFile);
-	return NULL;
-} /* end of pOpenDocument*/
-
-/*
- * vCloseDocument
+ * vFreeDocument - free a document by free-ing its parts
  */
 void
-vCloseDocument(FILE *pFile)
+vFreeDocument(void)
 {
-	DBG_MSG("vCloseDocument");
+	DBG_MSG("vFreeDocument");
 
 	/* Free the memory */
 	vDestroyTextBlockList();
@@ -842,44 +776,32 @@ vCloseDocument(FILE *pFile)
 	vDestroyStyleInfoList();
 	vDestroyFontInfoList();
 	vDestroyPicInfoList();
+	vDestroyPropModList();
 	vDestroyNotesInfoLists();
 	vDestroyFontTable();
-	/* Close the file */
-	(void)fclose(pFile);
-} /* end of vCloseDocument */
+} /* end of vFreeDocument */
 
 /*
  * Common part of the file checking functions
  */
 static BOOL
-bCheckBytes(const char *szFilename,
-	const unsigned char *aucBytes, size_t tBytes)
+bCheckBytes(FILE *pFile, const unsigned char *aucBytes, size_t tBytes)
 {
-	FILE	*pFile;
 	int	iIndex, iChar;
-	BOOL	bResult;
 
-	fail(aucBytes == NULL || tBytes == 0);
+	fail(pFile == NULL || aucBytes == NULL || tBytes == 0);
+
+	rewind(pFile);
 
-	if (szFilename == NULL || szFilename[0] == '\0') {
-		return FALSE;
-	}
-	pFile = fopen(szFilename, "rb");
-	if (pFile == NULL) {
-		return FALSE;
-	}
-	bResult = TRUE;
 	for (iIndex = 0; iIndex < (int)tBytes; iIndex++) {
 		iChar = getc(pFile);
 		if (iChar == EOF || iChar != (int)aucBytes[iIndex]) {
 			DBG_HEX(iChar);
 			DBG_HEX(aucBytes[iIndex]);
-			bResult = FALSE;
-			break;
+			return FALSE;
 		}
 	}
-	(void)fclose(pFile);
-	return bResult;
+	return TRUE;
 } /* end of bCheckBytes */
 
 /*
@@ -887,23 +809,22 @@ bCheckBytes(const char *szFilename,
  * document
  */
 BOOL
-bIsSupportedWordFile(const char *szFilename)
+bIsSupportedWordFile(FILE *pFile, long lFilesize)
 {
 	static unsigned char	aucBytes[] =
 		{ 0xd0, 0xcf, 0x11, 0xe0, 0xa1, 0xb1, 0x1a, 0xe1 };
-	long	lSize;
 
-	if (szFilename == NULL || szFilename[0] == '\0') {
-		DBG_MSG("No file given");
+	if (pFile == NULL || lFilesize < 0) {
+		DBG_MSG("No proper file given");
 		return FALSE;
 	}
-	lSize = lGetFilesize(szFilename);
-	if (lSize < 3 * BIG_BLOCK_SIZE || lSize % BIG_BLOCK_SIZE != 0) {
-		DBG_DEC(lSize);
-		DBG_MSG("Size mismatch");
+	if (lFilesize < (long)BIG_BLOCK_SIZE * 3 ||
+	    lFilesize % BIG_BLOCK_SIZE != 0) {
+		DBG_DEC(lFilesize);
+		DBG_MSG("File size mismatch");
 		return FALSE;
 	}
-	return bCheckBytes(szFilename, aucBytes, elementsof(aucBytes));
+	return bCheckBytes(pFile, aucBytes, elementsof(aucBytes));
 } /* end of bIsSupportedWordFile */
 
 /*
@@ -911,7 +832,7 @@ bIsSupportedWordFile(const char *szFilename)
  * document
  */
 BOOL
-bIsWord245File(const char *szFilename)
+bIsWord245File(FILE *pFile)
 {
 	static unsigned char	aucBytes[6][8] = {
 		{ 0x31, 0xbe, 0x00, 0x00, 0x00, 0xab, 0x00, 0x00 },
@@ -923,8 +844,10 @@ bIsWord245File(const char *szFilename)
 	};
 	int	iIndex;
 
+	DBG_MSG("bIsWord245File");
+
 	for (iIndex = 0; iIndex < (int)elementsof(aucBytes); iIndex++) {
-		if (bCheckBytes(szFilename,
+		if (bCheckBytes(pFile,
 				aucBytes[iIndex],
 				elementsof(aucBytes[iIndex]))) {
 			return TRUE;
@@ -937,23 +860,12 @@ bIsWord245File(const char *szFilename)
  * This function checks whether the given file is or is not a RTF document
  */
 BOOL
-bIsRtfFile(const char *szFilename)
+bIsRtfFile(FILE *pFile)
 {
 	static unsigned char	aucBytes[] =
 		{ '{', '\\', 'r', 't', 'f', '1' };
 
-	return bCheckBytes(szFilename, aucBytes, elementsof(aucBytes));
-} /* end of bIsRtfFile */
+	DBG_MSG("bIsRtfFile");
 
-/*
- * TRUE if the current file was made on an Apple Macintosh, otherwise FALSE.
- * This function hides the methode of how to find out from the rest of the
- * program.
- *
- * note: this should grow into a real function
- */
-BOOL
-bFileFromTheMac(void)
-{
-	return bMacFile;
-} /* end of bOriginatesOnMac */
+	return bCheckBytes(pFile, aucBytes, elementsof(aucBytes));
+} /* end of bIsRtfFile */
